@@ -1,4 +1,4 @@
-// Copyright (c) 2018 Intel Corporation
+// Copyright (c) 2018-2019 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -60,6 +60,17 @@ struct ThreadTaskInfo
     mfxU32            numDecodeTasksToCheck;
 };
 
+static void SetFrameType(mfxFrameSurface1 &surface_out)
+{
+    auto extFrameInfo = reinterpret_cast<mfxExtDecodedFrameInfo *>(GetExtendedBuffer(surface_out.Data.ExtParam, surface_out.Data.NumExtParam, MFX_EXTBUFF_DECODED_FRAME_INFO));
+    if (extFrameInfo == nullptr)
+        return;
+
+    // terms I/P/B frames not applicable for JPEG,
+    // so all frames marked as I
+    extFrameInfo->FrameType = MFX_FRAMETYPE_I;
+}
+
 class MFX_JPEG_Utility
 {
 public:
@@ -558,8 +569,8 @@ mfxStatus VideoDECODEMJPEG::QueryIOSurfInternal(VideoCORE *core, mfxVideoParam *
         std::swap(request->Info.CropY, request->Info.CropX);
     }
 
-    request->Info.Width = UMC::align_value<mfxU16>(request->Info.Width, 0x10);
-    request->Info.Height = UMC::align_value<mfxU16>(request->Info.Height,
+    request->Info.Width  = mfx::align2_value(request->Info.Width, 0x10);
+    request->Info.Height = mfx::align2_value(request->Info.Height,
         (request->Info.PicStruct == MFX_PICSTRUCT_PROGRESSIVE) ? 0x10 : 0x20);
 
     if (MFX_PLATFORM_SOFTWARE == platform)
@@ -568,11 +579,6 @@ mfxStatus VideoDECODEMJPEG::QueryIOSurfInternal(VideoCORE *core, mfxVideoParam *
     }
     else
     {
-    // large pictures are not supported on Linux so far
-    if ((request->Info.Width > 8192) || (request->Info.Height > 8192)) {
-        return MFX_ERR_UNSUPPORTED;
-    }
-
         eMFXHWType type = MFX_HW_UNKNOWN;
         if (platform == MFX_PLATFORM_HARDWARE)
         {
@@ -650,6 +656,8 @@ mfxStatus VideoDECODEMJPEG::DecodeFrameCheck(mfxBitstream *bs, mfxFrameSurface1
         if (!(*surface_out))
             return MFX_ERR_UNDEFINED_BEHAVIOR;
 
+        SetFrameType(**surface_out);
+
         (*surface_out)->Info.FrameId.ViewId = 0; // (mfxU16)pFrame->m_viewId;
 
         bool isShouldUpdate = !(m_vFirstPar.mfx.FrameInfo.AspectRatioH || m_vFirstPar.mfx.FrameInfo.AspectRatioW);
@@ -1002,7 +1010,8 @@ bool MFX_JPEG_Utility::IsNeedPartialAcceleration(VideoCORE * core, mfxVideoParam
 
 #if defined (MFX_VA_LINUX)
     if (par->mfx.FrameInfo.FourCC != MFX_FOURCC_NV12 &&
-        par->mfx.FrameInfo.FourCC != MFX_FOURCC_RGB4)
+        par->mfx.FrameInfo.FourCC != MFX_FOURCC_RGB4 &&
+        par->mfx.FrameInfo.FourCC != MFX_FOURCC_YUY2)
         return true;
 
     /* on Linux in case of multiscan turn off HW support until because some functionality is missed so far */
@@ -1010,7 +1019,7 @@ bool MFX_JPEG_Utility::IsNeedPartialAcceleration(VideoCORE * core, mfxVideoParam
         return true;
 
     if (core->GetHWType() == MFX_HW_APL
-        || core->GetHWType() == MFX_HW_CFL
+        || core->GetHWType() >= MFX_HW_KBL
        )
     {
         switch (par->mfx.FrameInfo.FourCC)
@@ -1024,7 +1033,6 @@ bool MFX_JPEG_Utility::IsNeedPartialAcceleration(VideoCORE * core, mfxVideoParam
                     return false;
                 else
                     return true;
-/*
             case MFX_FOURCC_YUY2:
             case MFX_FOURCC_UYVY:
                 if( par->mfx.JPEGColorFormat == MFX_JPEG_COLORFORMAT_YCbCr &&
@@ -1032,10 +1040,10 @@ bool MFX_JPEG_Utility::IsNeedPartialAcceleration(VideoCORE * core, mfxVideoParam
                   ) return false;
                 else
                     return true;
-*/
             case MFX_FOURCC_RGB4:
                 if ((par->mfx.JPEGColorFormat == MFX_JPEG_COLORFORMAT_RGB && par->mfx.JPEGChromaFormat != MFX_CHROMAFORMAT_YUV444) ||
                     (par->mfx.JPEGColorFormat == MFX_JPEG_COLORFORMAT_YCbCr && par->mfx.JPEGChromaFormat == MFX_CHROMAFORMAT_YUV422V) ||
+                    (par->mfx.JPEGColorFormat == MFX_JPEG_COLORFORMAT_YCbCr && par->mfx.JPEGChromaFormat == MFX_CHROMAFORMAT_MONOCHROME) ||
                     (par->mfx.JPEGColorFormat == MFX_JPEG_COLORFORMAT_YCbCr && par->mfx.JPEGChromaFormat == MFX_CHROMAFORMAT_YUV411))
                     return true;
                 else
@@ -1044,8 +1052,7 @@ bool MFX_JPEG_Utility::IsNeedPartialAcceleration(VideoCORE * core, mfxVideoParam
                 return true;
         }
     }
-
-    if (core->GetHWType() == MFX_HW_BDW || core->GetHWType() == MFX_HW_SCL)
+    else if (core->GetHWType() == MFX_HW_BDW || core->GetHWType() == MFX_HW_SCL)
     {
         if (par->mfx.JPEGColorFormat == MFX_JPEG_COLORFORMAT_YCbCr &&
             par->mfx.JPEGChromaFormat == MFX_CHROMAFORMAT_MONOCHROME)
@@ -1058,18 +1065,12 @@ bool MFX_JPEG_Utility::IsNeedPartialAcceleration(VideoCORE * core, mfxVideoParam
             par->mfx.JPEGChromaFormat == MFX_CHROMAFORMAT_YUV444)
             return true;
 
-        if (par->mfx.JPEGColorFormat == MFX_JPEG_COLORFORMAT_YCbCr &&
-            par->mfx.JPEGChromaFormat == MFX_CHROMAFORMAT_YUV420 &&
-            par->mfx.FrameInfo.FourCC == MFX_FOURCC_RGB4 &&
-            (par->mfx.FrameInfo.Width > 4096 || par->mfx.FrameInfo.Height > 4096))
-            return true;
-
         if (par->mfx.JPEGColorFormat == MFX_JPEG_COLORFORMAT_RGB &&
             par->mfx.JPEGChromaFormat == MFX_CHROMAFORMAT_YUV444 &&
             par->mfx.FrameInfo.FourCC != MFX_FOURCC_RGB4)
             return true;
     }
-    if (core->GetHWType() == MFX_HW_VLV)
+    else if (core->GetHWType() == MFX_HW_VLV)
     {
         if (par->mfx.JPEGColorFormat == MFX_JPEG_COLORFORMAT_RGB) return true;
     }
@@ -1178,9 +1179,8 @@ mfxStatus MFX_JPEG_Utility::Query(VideoCORE *core, mfxVideoParam *in, mfxVideoPa
             sts = MFX_ERR_UNSUPPORTED;
         }
 
-        out->mfx.FrameInfo.Width = UMC::align_value<mfxU16>(in->mfx.FrameInfo.Width, 0x10);
-
-        out->mfx.FrameInfo.Height = UMC::align_value<mfxU16>(in->mfx.FrameInfo.Height,
+        out->mfx.FrameInfo.Width  = mfx::align2_value(in->mfx.FrameInfo.Width, 0x10);
+        out->mfx.FrameInfo.Height = mfx::align2_value(in->mfx.FrameInfo.Height,
             (in->mfx.FrameInfo.PicStruct == MFX_PICSTRUCT_PROGRESSIVE) ? 0x10 : 0x20);
 
         if (in->mfx.FrameInfo.CropX <= out->mfx.FrameInfo.Width)
@@ -1495,7 +1495,6 @@ VideoDECODEMJPEGBase_HW::VideoDECODEMJPEGBase_HW()
     m_va = 0;
     m_dst = 0;
     m_numPic = 0;
-    m_pCc    = NULL;
     m_needVpp = false;
 }
 
@@ -1533,8 +1532,6 @@ mfxStatus VideoDECODEMJPEGBase_HW::Reset(mfxVideoParam *par)
 {
     m_pMJPEGVideoDecoder->Reset();
     m_numPic = 0;
-    delete m_pCc;
-    m_pCc    = 0;
 
     m_vPar = *par;
 
@@ -1571,9 +1568,6 @@ mfxStatus VideoDECODEMJPEGBase_HW::Close(void)
     m_numPic = 0;
     m_isOpaq = false;
 
-    delete m_pCc;
-    m_pCc    = 0;
-
     {
         std::lock_guard<std::mutex> guard(m_guard);
 
@@ -1630,38 +1624,6 @@ mfxU32 VideoDECODEMJPEGBase_HW::AdjustFrameAllocRequest(mfxFrameAllocRequest *re
     // set FourCC
     AdjustFourCC(&request->Info, info, hwType, vaType, &needVpp);
 
-    // WA for rotation of unaligned images
-    mfxU16 mcuWidth;
-    mfxU16 mcuHeight;
-    mfxU16 paddingWidth;
-    mfxU16 paddingHeight;
-
-    switch(info->JPEGChromaFormat)
-    {
-    case MFX_CHROMAFORMAT_YUV411:
-        mcuWidth  = 32;
-        mcuHeight = 8;
-        break;
-    case MFX_CHROMAFORMAT_YUV420:
-        mcuWidth  = 16;
-        mcuHeight = 16;
-        break;
-    case MFX_CHROMAFORMAT_YUV422H:
-        mcuWidth  = 16;
-        mcuHeight = 8;
-        break;
-    case MFX_CHROMAFORMAT_YUV422V:
-        mcuWidth  = 8;
-        mcuHeight = 16;
-        break;
-    case MFX_CHROMAFORMAT_YUV400:
-    case MFX_CHROMAFORMAT_YUV444:
-    default:
-        mcuWidth  = 8;
-        mcuHeight = 8;
-        break;
-    }
-
     if(info->Rotation == MFX_ROTATION_90 || info->Rotation == MFX_ROTATION_180 || info->Rotation == MFX_ROTATION_270)
     {
         needVpp = true;
@@ -1675,31 +1637,6 @@ mfxU32 VideoDECODEMJPEGBase_HW::AdjustFrameAllocRequest(mfxFrameAllocRequest *re
         std::swap(request->Info.CropY, request->Info.CropX);
     }
 
-
-    if(info->Rotation == MFX_ROTATION_90 || info->Rotation == MFX_ROTATION_270)
-        std::swap(mcuWidth, mcuHeight);
-
-    paddingWidth = (mfxU16)((2<<16) - request->Info.CropW) % mcuWidth;
-    paddingHeight = (mfxU16)((2<<16) - request->Info.CropH) % mcuHeight;
-
-    switch(info->Rotation)
-    {
-    case MFX_ROTATION_90:
-        request->Info.CropX = paddingWidth;
-        request->Info.CropW = request->Info.CropW + paddingWidth;
-        break;
-    case MFX_ROTATION_180:
-        request->Info.CropX = paddingWidth;
-        request->Info.CropW = request->Info.CropW + paddingWidth;
-        request->Info.CropY = paddingHeight;
-        request->Info.CropH = request->Info.CropH + paddingHeight;
-        break;
-    case MFX_ROTATION_270:
-        request->Info.CropY = paddingHeight;
-        request->Info.CropH = request->Info.CropH + paddingHeight;
-        break;
-    }
-
     m_needVpp = needVpp;
     if (m_needVpp)
     {
@@ -1716,23 +1653,14 @@ mfxU32 VideoDECODEMJPEGBase_HW::AdjustFrameAllocRequest(mfxFrameAllocRequest *re
     return m_needVpp ? 1 : 0;
 }
 
-void VideoDECODEMJPEGBase_HW::AdjustFourCC(mfxFrameInfo *requestFrameInfo, mfxInfoMFX *info, eMFXHWType hwType, eMFXVAType vaType, bool *needVpp)
+void VideoDECODEMJPEGBase_HW::AdjustFourCC(mfxFrameInfo *requestFrameInfo, mfxInfoMFX *info, eMFXHWType /*hwType*/, eMFXVAType /*vaType*/, bool *needVpp)
 {
-    (void)vaType;
-
     if (info->JPEGColorFormat == MFX_JPEG_COLORFORMAT_UNKNOWN || info->JPEGColorFormat == MFX_JPEG_COLORFORMAT_YCbCr)
     {
-        #if defined (MFX_VA_LINUX)
-            if (hwType == MFX_HW_APL) return;
-        #endif
         switch(info->JPEGChromaFormat)
         {
         case MFX_CHROMAFORMAT_MONOCHROME:
-            if (requestFrameInfo->FourCC == MFX_FOURCC_RGB4 &&
-                requestFrameInfo->CropW >= 128 &&
-                requestFrameInfo->CropH >= 128 &&
-                requestFrameInfo->CropW <= 4096 &&
-                requestFrameInfo->CropH <= 4096)
+            if (requestFrameInfo->FourCC == MFX_FOURCC_RGB4)
             {
                 requestFrameInfo->FourCC = MFX_FOURCC_NV12;
                 *needVpp = true;
@@ -1740,11 +1668,7 @@ void VideoDECODEMJPEGBase_HW::AdjustFourCC(mfxFrameInfo *requestFrameInfo, mfxIn
             break;
         case MFX_CHROMAFORMAT_YUV420:
             if (info->Rotation == MFX_ROTATION_0 &&
-                requestFrameInfo->FourCC == MFX_FOURCC_RGB4 &&
-                requestFrameInfo->CropW >= 128 &&
-                requestFrameInfo->CropH >= 128 &&
-                requestFrameInfo->CropW <= 4096 &&
-                requestFrameInfo->CropH <= 4096)
+                requestFrameInfo->FourCC == MFX_FOURCC_RGB4)
             {
                 requestFrameInfo->FourCC = MFX_FOURCC_NV12;
                 *needVpp = true;
@@ -1754,11 +1678,7 @@ void VideoDECODEMJPEGBase_HW::AdjustFourCC(mfxFrameInfo *requestFrameInfo, mfxIn
             break;
         case MFX_CHROMAFORMAT_YUV422H:
             if (info->Rotation == MFX_ROTATION_0 &&
-                requestFrameInfo->FourCC == MFX_FOURCC_RGB4 &&
-                requestFrameInfo->CropW >= 128 &&
-                requestFrameInfo->CropH >= 128 &&
-                requestFrameInfo->CropW <= 4096 &&
-                requestFrameInfo->CropH <= 4096)
+                requestFrameInfo->FourCC == MFX_FOURCC_RGB4)
             {
                 requestFrameInfo->FourCC = MFX_FOURCC_NV12;
                 *needVpp = true;
@@ -1768,11 +1688,10 @@ void VideoDECODEMJPEGBase_HW::AdjustFourCC(mfxFrameInfo *requestFrameInfo, mfxIn
             break;
         case MFX_CHROMAFORMAT_YUV444:
             if (info->Rotation == MFX_ROTATION_0 &&
-                requestFrameInfo->FourCC == MFX_FOURCC_RGB4 &&
-                requestFrameInfo->CropW >= 128 &&
-                requestFrameInfo->CropH >= 128 &&
-                requestFrameInfo->CropW <= 4096 &&
-                requestFrameInfo->CropH <= 4096)
+                (requestFrameInfo->FourCC == MFX_FOURCC_RGB4
+                 // for YUV444 jpeg, decoded stream contains 444P,
+                 // to get NV12 we must use VPP
+                 || requestFrameInfo->FourCC == MFX_FOURCC_NV12))
             {
                 requestFrameInfo->FourCC = MFX_FOURCC_NV12;
                 *needVpp = true;
@@ -1783,7 +1702,6 @@ void VideoDECODEMJPEGBase_HW::AdjustFourCC(mfxFrameInfo *requestFrameInfo, mfxIn
             break;
         }
     }
-    return;
 }
 
 
@@ -1801,7 +1719,6 @@ mfxStatus VideoDECODEMJPEGBase_HW::RunThread(void *params, mfxU32, mfxU32 )
             mfxSts = ((mfx_UMC_FrameAllocator_D3D_Converter *)m_FrameAllocator.get())->CheckPreparingToOutput(info->surface_out,
                                                                                                               info->dst,
                                                                                                               &m_vPar,
-                                                                                                              &m_pCc,
                                                                                                               (mfxU16)info->vppTaskID);
             if(mfxSts != MFX_TASK_DONE)
             {
@@ -1962,7 +1879,7 @@ mfxStatus VideoDECODEMJPEGBase_HW::FillEntryPoint(MFX_ENTRY_POINT *pEntryPoint,
         ((mfx_UMC_FrameAllocator_D3D_Converter *)m_FrameAllocator.get())->SetJPEGInfo(&info);
 
         // decoding is ready. prepare to output:
-        mfxStatus mfxSts = ((mfx_UMC_FrameAllocator_D3D_Converter *)m_FrameAllocator.get())->StartPreparingToOutput(surface_out, dst, &m_vPar, &m_pCc, &taskId, m_isOpaq);
+        mfxStatus mfxSts = ((mfx_UMC_FrameAllocator_D3D_Converter *)m_FrameAllocator.get())->StartPreparingToOutput(surface_out, dst, &m_vPar, &taskId, m_isOpaq);
         if (mfxSts < MFX_ERR_NONE)
         {
             return mfxSts;
