@@ -1,4 +1,4 @@
-// Copyright (c) 2018 Intel Corporation
+// Copyright (c) 2018-2019 Intel Corporation
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -36,17 +36,6 @@
 #define D3DDDIFMT_NV12 (D3DDDIFORMAT)(MFX_MAKEFOURCC('N', 'V', '1', '2'))
 #endif
 
-#define MFX_CHECK_WITH_ASSERT(EXPR, ERR) { assert(EXPR); MFX_CHECK(EXPR, ERR); }
-
-#define MFX_DESTROY_VABUFFER(vaBufferId, vaDisplay)    \
-do {                                               \
-    if ( vaBufferId != VA_INVALID_ID)              \
-    {                                              \
-        vaDestroyBuffer(vaDisplay, vaBufferId);    \
-        vaBufferId = VA_INVALID_ID;                \
-    }                                              \
-} while (0)
-
 #define CODEC_MPEG2_ENC_FCODE_X(width) ((width < 200) ? 3 : (width < 500) ? 4 : (width < 1400) ? 5 : 6)
 #define CODEC_MPEG2_ENC_FCODE_Y(fcodeX) ((fcodeX > 5) ? 5 : fcodeX)
 
@@ -464,20 +453,10 @@ mfxStatus VAAPIEncoder::Init(ExecuteBuffers* pExecuteBuffers, mfxU32 numRefFrame
 mfxStatus VAAPIEncoder::Init(ENCODE_FUNC func, ExecuteBuffers* pExecuteBuffers)
 {
     MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_HOTSPOTS, "VAAPIEncoder::Init");
-    m_initFrameWidth   = 16 * ((pExecuteBuffers->m_sps.FrameWidth + 15) >> 4);
 
-    if (pExecuteBuffers->m_sps.progressive_sequence)
-    {
-        m_initFrameHeight  = 16 * ((pExecuteBuffers->m_sps.FrameHeight + 15) >> 4);
-    }
-    else
-    {
-        m_initFrameHeight  = 32 * ((pExecuteBuffers->m_sps.FrameHeight + 31) >> 5);
-    }
-
-
-    //m_initFrameWidth   = pExecuteBuffers->m_sps.FrameWidth;
-    //m_initFrameHeight  = pExecuteBuffers->m_sps.FrameHeight;
+    m_initFrameWidth  = mfx::align2_value(pExecuteBuffers->m_sps.FrameWidth, 16);
+    m_initFrameHeight = mfx::align2_value(pExecuteBuffers->m_sps.FrameHeight,
+                            pExecuteBuffers->m_sps.progressive_sequence ? 16 : 32);
 
     //memset (&m_rawFrames, 0, sizeof(mfxRawFrames));
 
@@ -603,10 +582,10 @@ mfxStatus VAAPIEncoder::CreateContext(ExecuteBuffers* pExecuteBuffers, mfxU32 nu
 
     if (pExecuteBuffers->m_mbqp_data)
     {
-        m_mbqpDataBuffer.resize(((m_initFrameWidth / 16 + 63) & ~63) * ((m_initFrameHeight / 16 + 7) & ~7));
+        m_mbqpDataBuffer.resize(mfx::align2_value(m_initFrameWidth / 16, 64) * mfx::align2_value(m_initFrameHeight / 16, 8));
 
 //             if (IsOn(extOpt3->MBDisableSkipMap))
-//                 m_mb_noskip_buffer.resize(((m_width / 16 + 63) & ~63) * ((m_height / 16 + 7) & ~7));
+//                 m_mb_noskip_buffer.resize(mfx::align2_value(m_width / 16, 64) * mfx::align2_value(m_height / 16, 8)
     }
 
     return MFX_ERR_NONE;
@@ -875,8 +854,8 @@ mfxStatus VAAPIEncoder::FillSlices(ExecuteBuffers* pExecuteBuffers)
     m_numSliceGroups = 1;
 
     MFX_CHECK_WITH_ASSERT(height_in_mbs == pExecuteBuffers->m_pps.NumSlice, MFX_ERR_INCOMPATIBLE_VIDEO_PARAM);
-    m_numSliceGroups = 1;
 
+    mfxStatus sts;
     for (i = 0; i < height_in_mbs; i++) {
         ENCODE_SET_SLICE_HEADER_MPEG2&  ddiSlice = pExecuteBuffers->m_pSlice[i];
         assert(ddiSlice.NumMbsForSlice == width_in_mbs);
@@ -886,7 +865,9 @@ mfxStatus VAAPIEncoder::FillSlices(ExecuteBuffers* pExecuteBuffers)
         sliceParam->is_intra_slice     = ddiSlice.IntraSlice;
         // prevent GPU hang due to different scale_code in different slices
         sliceParam->quantiser_scale_code = pExecuteBuffers->m_pSlice[0].quantiser_scale_code;
-        MFX_DESTROY_VABUFFER(m_sliceParamBufferId[i], m_vaDisplay);
+
+        sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_sliceParamBufferId[i]);
+        MFX_CHECK_STS(sts);
     }
 
     vaSts = vaCreateBuffer(m_vaDisplay,
@@ -898,7 +879,6 @@ mfxStatus VAAPIEncoder::FillSlices(ExecuteBuffers* pExecuteBuffers)
         m_sliceParamBufferId);
     MFX_CHECK_WITH_ASSERT(VA_STATUS_SUCCESS == vaSts, MFX_ERR_DEVICE_FAILED);
 
-
     return MFX_ERR_NONE;
 } // mfxStatus VAAPIEncoder::FillSlices(ExecuteBuffers* pExecuteBuffers)
 
@@ -913,7 +893,9 @@ mfxStatus VAAPIEncoder::FillMiscParameterBuffer(ExecuteBuffers* pExecuteBuffers)
 
     miscQuality.PanicModeDisable = pExecuteBuffers->m_bDisablePanicMode ? 1 : 0;
 
-    MFX_DESTROY_VABUFFER(m_miscParamFpsId, m_vaDisplay);
+    mfxStatus sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_miscParamFpsId);
+    MFX_CHECK_STS(sts);
+
     vaSts = vaCreateBuffer(m_vaDisplay,
         m_vaContextEncode,
         VAEncMiscParameterBufferType,
@@ -923,7 +905,9 @@ mfxStatus VAAPIEncoder::FillMiscParameterBuffer(ExecuteBuffers* pExecuteBuffers)
         &m_miscParamFpsId);
     MFX_CHECK_WITH_ASSERT(VA_STATUS_SUCCESS == vaSts, MFX_ERR_DEVICE_FAILED);
 
-    MFX_DESTROY_VABUFFER(m_miscParamQualityId, m_vaDisplay);
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_miscParamQualityId);
+    MFX_CHECK_STS(sts);
+
     vaSts = vaCreateBuffer(m_vaDisplay,
         m_vaContextEncode,
         VAEncMiscParameterBufferType,
@@ -942,7 +926,9 @@ mfxStatus VAAPIEncoder::FillQualityLevelBuffer(ExecuteBuffers* pExecuteBuffers)
     VAEncMiscParameterBuffer *misc_param;
     VAEncMiscParameterBufferQualityLevel *quality_param;
 
-    MFX_DESTROY_VABUFFER(m_miscQualityParamId, m_vaDisplay);
+    mfxStatus sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_miscQualityParamId);
+    MFX_CHECK_STS(sts);
+
     vaSts = vaCreateBuffer(m_vaDisplay,
                    m_vaContextEncode,
                    VAEncMiscParameterBufferType,
@@ -966,7 +952,8 @@ mfxStatus VAAPIEncoder::FillQualityLevelBuffer(ExecuteBuffers* pExecuteBuffers)
 
     quality_param->quality_level = tu;
 
-    vaUnmapBuffer(m_vaDisplay, m_miscQualityParamId);
+    vaSts = vaUnmapBuffer(m_vaDisplay, m_miscQualityParamId);
+    MFX_CHECK_WITH_ASSERT(VA_STATUS_SUCCESS == vaSts, MFX_ERR_DEVICE_FAILED);
 
     return MFX_ERR_NONE;
 } // mfxStatus VAAPIEncoder::FillQualityLevelBuffer(ExecuteBuffers* pExecuteBuffers)
@@ -981,7 +968,9 @@ mfxStatus VAAPIEncoder::FillUserDataBuffer(mfxU8 *pUserData, mfxU32 userDataLen)
     mfxU32 correctedLength = MFX_MIN(userDataLen, UINT_MAX/8); // keep start code
     packedParamsBuffer.bit_length = correctedLength * 8;
 
-    MFX_DESTROY_VABUFFER(m_packedUserDataParamsId, m_vaDisplay);
+    mfxStatus sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_packedUserDataParamsId);
+    MFX_CHECK_STS(sts);
+
     vaSts = vaCreateBuffer(m_vaDisplay,
         m_vaContextEncode,
         VAEncPackedHeaderParameterBufferType,
@@ -991,7 +980,9 @@ mfxStatus VAAPIEncoder::FillUserDataBuffer(mfxU8 *pUserData, mfxU32 userDataLen)
         &m_packedUserDataParamsId);
     MFX_CHECK_WITH_ASSERT(VA_STATUS_SUCCESS == vaSts, MFX_ERR_DEVICE_FAILED);
 
-    MFX_DESTROY_VABUFFER(m_packedUserDataId, m_vaDisplay);
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_packedUserDataId);
+    MFX_CHECK_STS(sts);
+
     vaSts = vaCreateBuffer(m_vaDisplay,
         m_vaContextEncode,
         VAEncPackedHeaderDataBufferType,
@@ -1028,7 +1019,9 @@ mfxStatus VAAPIEncoder::FillVideoSignalInfoBuffer(ExecuteBuffers* pExecuteBuffer
     miscSeqInfo.display_horizontal_size = pExecuteBuffers->m_sps.FrameWidth;
     miscSeqInfo.display_vertical_size = pExecuteBuffers->m_sps.FrameHeight;
 
-    MFX_DESTROY_VABUFFER(m_miscParamSeqInfoId, m_vaDisplay);
+    mfxStatus sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_miscParamSeqInfoId);
+    MFX_CHECK_STS(sts);
+
     VAStatus vaSts = vaCreateBuffer(m_vaDisplay,
         m_vaContextEncode,
         VAEncMiscParameterBufferType,
@@ -1064,8 +1057,8 @@ mfxStatus VAAPIEncoder::FillMBQPBuffer(
     }
 
     //width(64byte alignment) height(8byte alignment)
-    mfxU32 bufW = ((width_in_mbs*4 + 63) & ~63);
-    mfxU32 bufH = ((height_in_mbs + 7) & ~7);
+    mfxU32 bufW = mfx::align2_value(width_in_mbs * 4, 64);
+    mfxU32 bufH = mfx::align2_value(height_in_mbs,     8);
 
     if (   mbqp && numMB >= static_cast<mfxU32>(width_in_mbs * height_in_mbs)
         && m_mbqpDataBuffer.size()*4 >= (bufW* bufH))
@@ -1077,12 +1070,12 @@ mfxStatus VAAPIEncoder::FillMBQPBuffer(
                 m_mbqpDataBuffer[mbRow * (bufW/4) + mbCol].qp_y = mbqp[mbRow * width_in_mbs + mbCol];
     }
 
-
-    MFX_DESTROY_VABUFFER(m_mbqpBufferId, m_vaDisplay);
+    mfxStatus sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_mbqpBufferId);
+    MFX_CHECK_STS(sts);
 
     vaSts = vaCreateBuffer(m_vaDisplay,
         m_vaContextEncode,
-        (VABufferType)VAEncQPBufferType,
+        VAEncQPBufferType,
         bufW,
         bufH,//m_mbqpDataBuffer.size(),
         &m_mbqpDataBuffer[0],
@@ -1114,7 +1107,9 @@ mfxStatus VAAPIEncoder::FillSkipFrameBuffer(mfxU8 skipFlag)
     skipParam.num_skip_frames = 0; // unused in MPEG2
     skipParam.size_skip_frames = 0; // unused in MPEG2
 
-    MFX_DESTROY_VABUFFER(m_miscParamSkipFrameId, m_vaDisplay);
+    mfxStatus sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_miscParamSkipFrameId);
+    MFX_CHECK_STS(sts);
+
     vaSts = vaCreateBuffer(m_vaDisplay,
         m_vaContextEncode,
         VAEncMiscParameterBufferType,
@@ -1134,10 +1129,8 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
     mfxStatus mfxSts;
     VAStatus vaSts;
 
-    const mfxU32            NumCompBuffer = 15;
-    std::vector<VABufferID> configBuffers(NumCompBuffer, VA_INVALID_ID);
-
-    mfxU16 buffersCount = 0;
+    std::vector<VABufferID> configBuffers;
+    configBuffers.reserve(15);
 
     if (pExecuteBuffers->m_bAddSPS)
     {
@@ -1145,8 +1138,9 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
         Zero(m_vaSpsBuf);
         FillSps(pExecuteBuffers, m_vaSpsBuf);
 
+        mfxSts = CheckAndDestroyVAbuffer(m_vaDisplay, m_spsBufferId);
+        MFX_CHECK_STS(mfxSts);
 
-        MFX_DESTROY_VABUFFER(m_spsBufferId, m_vaDisplay);
         vaSts = vaCreateBuffer(m_vaDisplay,
             m_vaContextEncode,
             VAEncSequenceParameterBufferType,
@@ -1157,7 +1151,7 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
         MFX_CHECK_WITH_ASSERT(VA_STATUS_SUCCESS == vaSts, MFX_ERR_DEVICE_FAILED);
 
         if (m_spsBufferId != VA_INVALID_ID)
-            configBuffers[buffersCount++] = m_spsBufferId;
+            configBuffers.push_back(m_spsBufferId);
 
         // mfxExtVideoSignalInfo present - insert only with SPS
         if (pExecuteBuffers->m_bAddDisplayExt)
@@ -1166,7 +1160,7 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
             MFX_CHECK(mfxSts == MFX_ERR_NONE, MFX_ERR_DEVICE_FAILED);
 
             if (m_miscParamSeqInfoId != VA_INVALID_ID)
-                configBuffers[buffersCount++] = m_miscParamSeqInfoId;
+                configBuffers.push_back(m_miscParamSeqInfoId);
         }
 
         pExecuteBuffers->m_bAddSPS = 0;
@@ -1177,8 +1171,9 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
              pExecuteBuffers->m_quantMatrix.load_chroma_intra_quantiser_matrix ||
              pExecuteBuffers->m_quantMatrix.load_chroma_non_intra_quantiser_matrix))
         {
+            mfxSts = CheckAndDestroyVAbuffer(m_vaDisplay, m_qmBufferId);
+            MFX_CHECK_STS(mfxSts);
 
-            MFX_DESTROY_VABUFFER(m_qmBufferId, m_vaDisplay);
             vaSts = vaCreateBuffer(m_vaDisplay,
                 m_vaContextEncode,
                 VAQMatrixBufferType,
@@ -1189,7 +1184,7 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
             MFX_CHECK_WITH_ASSERT(VA_STATUS_SUCCESS == vaSts, MFX_ERR_DEVICE_FAILED);
 
             if (m_qmBufferId != VA_INVALID_ID)
-                configBuffers[buffersCount++] = m_qmBufferId;
+                configBuffers.push_back(m_qmBufferId);
         }
     }
 
@@ -1210,7 +1205,8 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
     }
     else
     {
-        MFX_DESTROY_VABUFFER(m_miscParamSkipFrameId, m_vaDisplay);
+        mfxSts = CheckAndDestroyVAbuffer(m_vaDisplay, m_miscParamSkipFrameId);
+        MFX_CHECK_STS(mfxSts);
     }
 
     mfxSts = FillSlices(pExecuteBuffers);
@@ -1220,7 +1216,9 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
     MFX_CHECK_WITH_ASSERT(pExecuteBuffers->m_idxBs >= 0  && pExecuteBuffers->m_idxBs < m_bsQueue.size(), MFX_ERR_DEVICE_FAILED);
     m_vaPpsBuf.coded_buf = m_bsQueue[pExecuteBuffers->m_idxBs].surface;
 
-    MFX_DESTROY_VABUFFER(m_ppsBufferId, m_vaDisplay);
+    mfxSts = CheckAndDestroyVAbuffer(m_vaDisplay, m_ppsBufferId);
+    MFX_CHECK_STS(mfxSts);
+
     vaSts = vaCreateBuffer(m_vaDisplay,
         m_vaContextEncode,
         VAEncPictureParameterBufferType,
@@ -1231,19 +1229,19 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
     MFX_CHECK_WITH_ASSERT(VA_STATUS_SUCCESS == vaSts, MFX_ERR_DEVICE_FAILED);
 
     if (m_ppsBufferId != VA_INVALID_ID)
-        configBuffers[buffersCount++] = m_ppsBufferId;
+        configBuffers.push_back(m_ppsBufferId);
 
     if (m_miscParamFpsId != VA_INVALID_ID)
-        configBuffers[buffersCount++] = m_miscParamFpsId;
+        configBuffers.push_back(m_miscParamFpsId);
 
     if (m_miscParamQualityId != VA_INVALID_ID)
-        configBuffers[buffersCount++] = m_miscParamQualityId;
+        configBuffers.push_back(m_miscParamQualityId);
 
     if (m_miscQualityParamId != VA_INVALID_ID)
-        configBuffers[buffersCount++] = m_miscQualityParamId;
+        configBuffers.push_back(m_miscQualityParamId);
 
     if (m_miscParamSkipFrameId != VA_INVALID_ID)
-        configBuffers[buffersCount++] = m_miscParamSkipFrameId;
+        configBuffers.push_back(m_miscParamSkipFrameId);
 
     if (pUserData && userDataLen > 0)
     {
@@ -1251,10 +1249,10 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
         MFX_CHECK(mfxSts == MFX_ERR_NONE, MFX_ERR_DEVICE_FAILED);
 
         if (m_packedUserDataParamsId != VA_INVALID_ID)
-            configBuffers[buffersCount++] = m_packedUserDataParamsId;
+            configBuffers.push_back(m_packedUserDataParamsId);
 
         if (m_packedUserDataId != VA_INVALID_ID)
-            configBuffers[buffersCount++] = m_packedUserDataId;
+            configBuffers.push_back(m_packedUserDataId);
     }
 
     bool isMBQP = pExecuteBuffers->m_mbqp_data && (pExecuteBuffers->m_mbqp_data[0] != 0);
@@ -1264,7 +1262,7 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
         MFX_CHECK(mfxSts == MFX_ERR_NONE, MFX_ERR_DEVICE_FAILED);
 
         if (m_mbqpBufferId != VA_INVALID_ID)
-            configBuffers[buffersCount++] = m_mbqpBufferId;
+            configBuffers.push_back(m_mbqpBufferId);
 
         pExecuteBuffers->m_mbqp_data[0] = 0;
     }
@@ -1290,8 +1288,8 @@ mfxStatus VAAPIEncoder::Execute(ExecuteBuffers* pExecuteBuffers, mfxU32 funcId,
             MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_EXTCALL, "vaRenderPicture(buf)");
             vaSts = vaRenderPicture(m_vaDisplay,
                 m_vaContextEncode,
-                &configBuffers[0],
-                buffersCount);
+                configBuffers.data(),
+                configBuffers.size());
             MFX_CHECK_WITH_ASSERT(VA_STATUS_SUCCESS == vaSts, MFX_ERR_DEVICE_FAILED);
         }
 
@@ -1435,24 +1433,44 @@ mfxStatus VAAPIEncoder::Close()
     delete [] m_pMiscParamsSkipFrame;
     m_pMiscParamsSkipFrame = 0;
 
-    MFX_DESTROY_VABUFFER(m_spsBufferId, m_vaDisplay);
-    MFX_DESTROY_VABUFFER(m_qmBufferId,  m_vaDisplay);
-    MFX_DESTROY_VABUFFER(m_ppsBufferId, m_vaDisplay);
+    mfxStatus sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_spsBufferId);
+    std::ignore = MFX_STS_TRACE(sts);
+
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_qmBufferId);
+    std::ignore = MFX_STS_TRACE(sts);
+
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_ppsBufferId);
+    std::ignore = MFX_STS_TRACE(sts);
+
     for (mfxU32 i = 0; i < MAX_SLICES; i++)
     {
-        MFX_DESTROY_VABUFFER(m_sliceParamBufferId[i], m_vaDisplay);
+        sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_sliceParamBufferId[i]);
+        std::ignore = MFX_STS_TRACE(sts);
     }
 
-    MFX_DESTROY_VABUFFER(m_miscParamFpsId,       m_vaDisplay);
-    MFX_DESTROY_VABUFFER(m_miscParamQualityId,   m_vaDisplay);
-    MFX_DESTROY_VABUFFER(m_miscQualityParamId,   m_vaDisplay);
-    MFX_DESTROY_VABUFFER(m_miscParamSeqInfoId,   m_vaDisplay);
-    MFX_DESTROY_VABUFFER(m_miscParamSkipFrameId, m_vaDisplay);
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_miscParamFpsId);
+    std::ignore = MFX_STS_TRACE(sts);
 
-    MFX_DESTROY_VABUFFER(m_packedUserDataParamsId, m_vaDisplay);
-    MFX_DESTROY_VABUFFER(m_packedUserDataId,       m_vaDisplay);
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_miscParamQualityId);
+    std::ignore = MFX_STS_TRACE(sts);
 
-    MFX_DESTROY_VABUFFER(m_mbqpBufferId, m_vaDisplay);
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_miscQualityParamId);
+    std::ignore = MFX_STS_TRACE(sts);
+
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_miscParamSeqInfoId);
+    std::ignore = MFX_STS_TRACE(sts);
+
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_miscParamSkipFrameId);
+    std::ignore = MFX_STS_TRACE(sts);
+
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_packedUserDataParamsId);
+    std::ignore = MFX_STS_TRACE(sts);
+
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_packedUserDataId);
+    std::ignore = MFX_STS_TRACE(sts);
+
+    sts = CheckAndDestroyVAbuffer(m_vaDisplay, m_mbqpBufferId);
+    std::ignore = MFX_STS_TRACE(sts);
 
     if (m_allocResponseMB.NumFrameActual != 0)
     {
@@ -1472,13 +1490,15 @@ mfxStatus VAAPIEncoder::Close()
     {
         MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_EXTCALL, "vaDestroyContext");
 
-        vaDestroyContext(m_vaDisplay, m_vaContextEncode);
+        VAStatus vaSts = vaDestroyContext(m_vaDisplay, m_vaContextEncode);
+        std::ignore = MFX_STS_TRACE(vaSts);
         m_vaContextEncode = VA_INVALID_ID;
     }
 
     if (m_vaConfig != VA_INVALID_ID)
     {
-        vaDestroyConfig(m_vaDisplay, m_vaConfig);
+        VAStatus vaSts = vaDestroyConfig(m_vaDisplay, m_vaConfig);
+        std::ignore = MFX_STS_TRACE(vaSts);
         m_vaConfig = VA_INVALID_ID;
     }
 
