@@ -1,4 +1,4 @@
-// Copyright (c) 2017 Intel Corporation
+// Copyright (c) 2017-2019 Intel Corporation
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -920,9 +920,7 @@ mfxStatus _mfxSession_1_10::InitEx(mfxInitParam& par)
         if (pCmCore)
         {
             mfxRes = pCmCore->SetCmCopyStatus(false);
-        }
-        if (MFX_ERR_NONE != mfxRes) {
-            return mfxRes;
+            MFX_CHECK_STS(mfxRes);
         }
     }
 
