@@ -1,4 +1,4 @@
-// Copyright (c) 2018 Intel Corporation
+// Copyright (c) 2018-2019 Intel Corporation
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -636,8 +636,8 @@ namespace MfxHwVideoProcessing
                 m_frcRational[VPP_IN]  = frcRational[VPP_IN];
                 m_frcRational[VPP_OUT] = frcRational[VPP_OUT];
 
-                m_minDeltaTime = MFX_MIN((__UINT64) (m_frcRational[VPP_IN].FrameRateExtD * MFX_TIME_STAMP_FREQUENCY) / (2 * m_frcRational[VPP_IN].FrameRateExtN),
-                    (__UINT64) (m_frcRational[VPP_OUT].FrameRateExtD * MFX_TIME_STAMP_FREQUENCY) / (2 * m_frcRational[VPP_OUT].FrameRateExtN));
+                m_minDeltaTime = MFX_MIN(uint64_t(m_frcRational[VPP_IN].FrameRateExtD  * MFX_TIME_STAMP_FREQUENCY) / (2 * m_frcRational[VPP_IN].FrameRateExtN),
+                                         uint64_t(m_frcRational[VPP_OUT].FrameRateExtD * MFX_TIME_STAMP_FREQUENCY) / (2 * m_frcRational[VPP_OUT].FrameRateExtN));
             }
 
             mfxStatus DoCpuFRC_AndUpdatePTS(
@@ -908,6 +908,8 @@ namespace MfxHwVideoProcessing
             mfxFrameSurface1 *pInputSurface,
             mfxFrameSurface1 *pOutputSurface);
 
+        bool UseCopyPassThrough(const DdiTask *pTask) const;
+
         mfxStatus PreWorkOutSurface(ExtSurface & output);
         mfxStatus PreWorkInputSurface(std::vector<ExtSurface> & surfQueue);
 
@@ -961,7 +963,7 @@ namespace MfxHwVideoProcessing
         mfxU32        m_frame_num;
         mfxStatus     m_critical_error;
 
-        // Not an auto_ptr anymore since core owns create/delete semantic now.
+        // Not a smart pointer anymore since core owns create/delete semantic now.
         VPPHWResMng * m_ddi;
         bool          m_bMultiView;
 
@@ -1005,6 +1007,13 @@ namespace MfxHwVideoProcessing
         public:
             void SetCmDevice(CmDevice * device) { m_pCmDevice = device; }
     };
+
+    class VpUnsupportedError : public std::exception
+    {
+    public:
+        VpUnsupportedError() : std::exception() { assert(!"VpUnsupportedError"); }
+    };
+
 }; // namespace MfxHwVideoProcessing
 
 
