@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2018 Intel Corporation
+// Copyright (c) 2017-2019 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -216,18 +216,17 @@ mfxStatus CommonCORE::DefaultAllocFrames(mfxFrameAllocRequest *request, mfxFrame
         (request->Type & MFX_MEMTYPE_DXVA2_PROCESSOR_TARGET))
         // should be SW
         return MFX_ERR_UNSUPPORTED;
-    else //if (request->Type & MFX_MEMTYPE_INTERNAL_FRAME) // TBD - only internal frames can be allocated
+
+    //if (request->Type & MFX_MEMTYPE_INTERNAL_FRAME) // TBD - only internal frames can be allocated
     {
         mfxBaseWideFrameAllocator* pAlloc = GetAllocatorByReq(request->Type);
         // VPP, ENC, PAK can request frames for several times
-        if (!pAlloc)
-        {
-            m_pcAlloc.reset(new mfxWideSWFrameAllocator(request->Type));
-            pAlloc = m_pcAlloc.get();
-        }
-        else
+        if (pAlloc)
             return MFX_ERR_MEMORY_ALLOC;
 
+        m_pcAlloc.reset(new mfxWideSWFrameAllocator(request->Type));
+        pAlloc = m_pcAlloc.get();
+
         // set frame allocator
         pAlloc->frameAllocator.pthis = pAlloc;
         // set buffer allocator for current frame single allocator
@@ -238,7 +237,7 @@ mfxStatus CommonCORE::DefaultAllocFrames(mfxFrameAllocRequest *request, mfxFrame
         MFX_CHECK_STS(sts);
     }
     ++m_NumAllocators;
-    m_pcAlloc.pop();
+    m_pcAlloc.release();
     return sts;
 }
 mfxStatus CommonCORE::LockFrame(mfxHDL mid, mfxFrameData *ptr)
@@ -412,7 +411,7 @@ mfxStatus CommonCORE::InternalFreeFrames(mfxFrameAllocResponse *response)
             ctbl_it = m_CTbl.find(response->mids[i]);
             if (m_CTbl.end() == ctbl_it)
                 return MFX_ERR_INVALID_HANDLE;
-            m_pMemId.get()[i] = ctbl_it->second.InternalMid;
+            m_pMemId[i] = ctbl_it->second.InternalMid;
             pAlloc = GetAllocatorAndMid(extMem);
             // all frames should be allocated by one allocator
             if ((IsDefaultMem != ctbl_it->second.isDefaultMem)||
@@ -443,7 +442,7 @@ mfxStatus CommonCORE::InternalFreeFrames(mfxFrameAllocResponse *response)
                 m_AllocatorQueue.erase(response->mids[i]);
             m_CTbl.erase(ctbl_it);
         }
-        m_pMemId.reset(0);
+        m_pMemId.reset();
         // we sure about response->mids
         delete[] response->mids;
         response->mids = 0;
@@ -647,11 +646,11 @@ mfxStatus CommonCORE::RegisterMids(mfxFrameAllocResponse *response, mfxU16 memTy
         // set render target memory description
         ds.memType = memType;
         m_CTbl.insert(pair<mfxMemId, MemDesc>(mId, ds));
-        m_pMemId.get()[i] = mId;
+        m_pMemId[i] = mId;
     }
     m_RespMidQ.insert(pair<mfxMemId*, mfxMemId*>(m_pMemId.get(), response->mids));
-    response->mids = m_pMemId.get();
-    m_pMemId.pop();
+    response->mids = m_pMemId.release();
+
     return  MFX_ERR_NONE;
 }
 
@@ -1167,10 +1166,9 @@ mfxStatus CommonCORE::DoFastCopy(mfxFrameSurface1 *dst, mfxFrameSurface1 *src)
     uint32_t srcPitch;
     uint32_t dstPitch;
 
-    FastCopy *pFastCopy = m_pFastCopy.get();
-    if(!pFastCopy){
+    if(!m_pFastCopy)
+    {
         m_pFastCopy.reset(new FastCopy());
-        pFastCopy = m_pFastCopy.get();
     }
 
     pDst = dst->Data.Y;
@@ -1188,7 +1186,7 @@ mfxStatus CommonCORE::DoFastCopy(mfxFrameSurface1 *dst, mfxFrameSurface1 *src)
     {
     case MFX_FOURCC_NV12:
 
-        sts = pFastCopy->Copy(pDst, dstPitch, pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
+        sts = m_pFastCopy->Copy(pDst, dstPitch, pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
 
         roi.height >>= 1;
 
@@ -1200,13 +1198,13 @@ mfxStatus CommonCORE::DoFastCopy(mfxFrameSurface1 *dst, mfxFrameSurface1 *src)
             return MFX_ERR_NULL_PTR;
         }
 
-        sts = pFastCopy->Copy(pDst, dstPitch, pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
+        sts = m_pFastCopy->Copy(pDst, dstPitch, pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
 
         break;
 
     case MFX_FOURCC_YV12:
 
-        sts = pFastCopy->Copy(pDst, dstPitch, pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
+        sts = m_pFastCopy->Copy(pDst, dstPitch, pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
 
         roi.height >>= 1;
 
@@ -1223,7 +1221,7 @@ mfxStatus CommonCORE::DoFastCopy(mfxFrameSurface1 *dst, mfxFrameSurface1 *src)
         srcPitch >>= 1;
         dstPitch >>= 1;
 
-        sts = pFastCopy->Copy((mfxU8 *)pDst, dstPitch, (mfxU8 *)pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
+        sts = m_pFastCopy->Copy((mfxU8 *)pDst, dstPitch, (mfxU8 *)pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
 
         pSrc = src->Data.V;
         pDst = dst->Data.V;
@@ -1233,7 +1231,7 @@ mfxStatus CommonCORE::DoFastCopy(mfxFrameSurface1 *dst, mfxFrameSurface1 *src)
             return MFX_ERR_NULL_PTR;
         }
 
-        sts = pFastCopy->Copy((mfxU8 *)pDst, dstPitch, (mfxU8 *)pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
+        sts = m_pFastCopy->Copy((mfxU8 *)pDst, dstPitch, (mfxU8 *)pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
 
         break;
 
@@ -1241,13 +1239,13 @@ mfxStatus CommonCORE::DoFastCopy(mfxFrameSurface1 *dst, mfxFrameSurface1 *src)
 
         roi.width *= 2;
 
-        sts = pFastCopy->Copy(pDst, dstPitch, pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
+        sts = m_pFastCopy->Copy(pDst, dstPitch, pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
 
         break;
 
     case MFX_FOURCC_P8:
 
-        sts = pFastCopy->Copy(pDst, dstPitch, pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
+        sts = m_pFastCopy->Copy(pDst, dstPitch, pSrc, srcPitch, roi, COPY_SYS_TO_SYS);
 
         break;
 
@@ -1290,8 +1288,7 @@ mfxStatus CoreDoSWFastCopy(mfxFrameSurface1 *pDst, mfxFrameSurface1 *pSrc, int c
             else
                 lshift = (uint8_t)(16 - pDst->Info.BitDepthLuma);
 
-            roi.width <<= 1;
-
+            // CopyAndShift operates with 2-byte words, no need to multiply width by 2
             sts = FastCopy::CopyAndShift((mfxU16*)(pDst->Data.Y), dstPitch, (mfxU16 *)pSrc->Data.Y, srcPitch, roi, lshift, rshift, copyFlag);
             MFX_CHECK_STS(sts);
 
@@ -1427,9 +1424,9 @@ mfxStatus CoreDoSWFastCopy(mfxFrameSurface1 *pDst, mfxFrameSurface1 *pSrc, int c
         MFX_CHECK_NULL_PTR1(pSrc->Data.Y);
 
         //we use 8u copy, so we need to increase ROI to handle 16 bit samples
-        roi.width *= 4;
         if (pSrc->Info.Shift != pDst->Info.Shift)
         {
+            roi.width *= 2; // CopyAndShift operates with 2-byte words
             mfxU8 lshift = 0;
             mfxU8 rshift = 0;
             if(pSrc->Info.Shift != 0)
@@ -1440,7 +1437,10 @@ mfxStatus CoreDoSWFastCopy(mfxFrameSurface1 *pDst, mfxFrameSurface1 *pSrc, int c
             sts = FastCopy::CopyAndShift((mfxU16*)(pDst->Data.Y), dstPitch, (mfxU16 *)pSrc->Data.Y, srcPitch, roi, lshift, rshift, copyFlag);
         }
         else
+        {
+            roi.width *= 4;
             sts = FastCopy::Copy(pDst->Data.Y, dstPitch, pSrc->Data.Y, srcPitch, roi, copyFlag);
+        }
 
         MFX_CHECK_STS(sts);
         break;
@@ -1719,9 +1719,8 @@ bool CommonCORE::IsOpaqSurfacesAlreadyMapped(mfxFrameSurface1 **pOpaqueSurface,
         // consistent already checked in CheckOpaqueRequest function
         if (oqp_it != m_OpqTbl.end())
         {
-            m_pMemId.reset(new mfxMemId[NumOpaqueSurface]);
-            response->mids = m_pMemId.get();
-            m_pMemId.pop();
+            m_pMemId.reset();
+            response->mids = new mfxMemId[NumOpaqueSurface];
 
             for (; i < NumOpaqueSurface; i++)
             {
