@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2018 Intel Corporation
+// Copyright (c) 2017-2019 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -32,6 +32,7 @@
 #include "mfx_common_decode_int.h"
 #include "mfx_enc_common.h"
 #include "mfxfei.h"
+#include "libmfx_core_hw.h"
 #include "umc_va_fei.h"
 
 #include "cm_mem_copy.h"
@@ -272,6 +273,30 @@ typedef struct {
     { 0x3EA3, MFX_HW_CFL, MFX_GT2 },
     { 0x3EA4, MFX_HW_CFL, MFX_GT1 },
 
+
+    /* CML GT1 */
+    { 0x9b21, MFX_HW_CFL, MFX_GT1 },
+    { 0x9baa, MFX_HW_CFL, MFX_GT1 },
+    { 0x9bab, MFX_HW_CFL, MFX_GT1 },
+    { 0x9bac, MFX_HW_CFL, MFX_GT1 },
+    { 0x9ba0, MFX_HW_CFL, MFX_GT1 },
+    { 0x9ba5, MFX_HW_CFL, MFX_GT1 },
+    { 0x9ba8, MFX_HW_CFL, MFX_GT1 },
+    { 0x9ba4, MFX_HW_CFL, MFX_GT1 },
+    { 0x9ba2, MFX_HW_CFL, MFX_GT1 },
+
+    /* CML GT2 */
+    { 0x9b41, MFX_HW_CFL, MFX_GT2 },
+    { 0x9bca, MFX_HW_CFL, MFX_GT2 },
+    { 0x9bcb, MFX_HW_CFL, MFX_GT2 },
+    { 0x9bcc, MFX_HW_CFL, MFX_GT2 },
+    { 0x9bc0, MFX_HW_CFL, MFX_GT2 },
+    { 0x9bc5, MFX_HW_CFL, MFX_GT2 },
+    { 0x9bc8, MFX_HW_CFL, MFX_GT2 },
+    { 0x9bc4, MFX_HW_CFL, MFX_GT2 },
+    { 0x9bc2, MFX_HW_CFL, MFX_GT2 },
+
+
     /* CNL */
     { 0x5A51, MFX_HW_CNL, MFX_GT2 },
     { 0x5A52, MFX_HW_CNL, MFX_GT2 },
@@ -283,16 +308,20 @@ typedef struct {
     { 0x5A49, MFX_HW_CNL, MFX_GT2 },
 
     /* ICL LP */
+    { 0xFF05, MFX_HW_ICL_LP, MFX_GT1 },
     { 0x8A50, MFX_HW_ICL_LP, MFX_GT2 },
     { 0x8A51, MFX_HW_ICL_LP, MFX_GT2 },
-    { 0x8A5C, MFX_HW_ICL_LP, MFX_GT2 },
-    { 0x8A5D, MFX_HW_ICL_LP, MFX_GT2 },
     { 0x8A52, MFX_HW_ICL_LP, MFX_GT2 },
-    { 0x8A5A, MFX_HW_ICL_LP, MFX_GT2 },
-    { 0x8A5B, MFX_HW_ICL_LP, MFX_GT2 },
-    { 0x8A71, MFX_HW_ICL_LP, MFX_GT1 },
-    { 0x8A70, MFX_HW_ICL_LP, MFX_GT1 }
-
+    { 0x8A53, MFX_HW_ICL_LP, MFX_GT2 },
+    { 0x8A56, MFX_HW_ICL_LP, MFX_GT1 },
+    { 0x8A57, MFX_HW_ICL_LP, MFX_GT1 },
+    { 0x8A58, MFX_HW_ICL_LP, MFX_GT1 },
+    { 0x8A59, MFX_HW_ICL_LP, MFX_GT1 },
+    { 0x8A5A, MFX_HW_ICL_LP, MFX_GT1 },
+    { 0x8A5B, MFX_HW_ICL_LP, MFX_GT1 },
+    { 0x8A5C, MFX_HW_ICL_LP, MFX_GT1 },
+    { 0x8A5D, MFX_HW_ICL_LP, MFX_GT1 },
+    { 0x8A71, MFX_HW_ICL_LP, MFX_GT1 }  // GT05, but 1 ok in this context
  };
 
 /* END: IOCTLs definitions */
@@ -376,7 +405,7 @@ VAAPIVideoCORE::~VAAPIVideoCORE()
 {
     if (m_bCmCopy)
     {
-        m_pCmCopy.get()->Release();
+        m_pCmCopy->Release();
         m_bCmCopy = false;
     }
 
@@ -400,6 +429,20 @@ VAAPIVideoCORE::GetHandle(
     MFX_CHECK_NULL_PTR1(handle);
     UMC::AutomaticUMCMutex guard(m_guard);
 
+#if defined (MFX_ENABLE_CPLIB)
+    if (MFX_HANDLE_VA_CONTEXT_ID == (mfxU32)type )
+    {
+        if (m_VAContextHandle != (mfxHDL)VA_INVALID_ID)
+        {
+            *handle = m_VAContextHandle;
+            return MFX_ERR_NONE;
+        }
+        // not exist handle yet
+        else
+            return MFX_ERR_NOT_FOUND;
+    }
+    else
+#endif
         return CommonCORE::GetHandle(type, handle);
 
 } // mfxStatus VAAPIVideoCORE::GetHandle(mfxHandleType type, mfxHDL *handle)
@@ -415,6 +458,24 @@ VAAPIVideoCORE::SetHandle(
     {
         switch ((mfxU32)type)
         {
+#if defined (MFX_ENABLE_CPLIB)
+        case MFX_HANDLE_VA_CONFIG_ID:
+            // if device manager already set
+            if (m_VAConfigHandle != (mfxHDL)VA_INVALID_ID)
+                return MFX_ERR_UNDEFINED_BEHAVIOR;
+            // set external handle
+            m_VAConfigHandle = hdl;
+            m_KeepVAState = true;
+            break;
+        case MFX_HANDLE_VA_CONTEXT_ID:
+            // if device manager already set
+            if (m_VAContextHandle != (mfxHDL)VA_INVALID_ID)
+                return MFX_ERR_UNDEFINED_BEHAVIOR;
+            // set external handle
+            m_VAContextHandle = hdl;
+            m_KeepVAState = true;
+            break;
+#endif
         default:
             mfxStatus sts = CommonCORE::SetHandle(type, hdl);
             MFX_CHECK_STS(sts);
@@ -475,19 +536,19 @@ VAAPIVideoCORE::AllocFrames(
         if (!m_bCmCopy && m_bCmCopyAllowed && isNeedCopy && m_Display)
         {
             m_pCmCopy.reset(new CmCopyWrapper);
-            if (!m_pCmCopy.get()->GetCmDevice(m_Display)){
+            if (!m_pCmCopy->GetCmDevice(m_Display)){
                 m_bCmCopy = false;
                 m_bCmCopyAllowed = false;
-                m_pCmCopy.get()->Release();
+                m_pCmCopy->Release();
                 m_pCmCopy.reset();
             }else{
-                sts = m_pCmCopy.get()->Initialize(GetHWType());
+                sts = m_pCmCopy->Initialize(GetHWType());
                 MFX_CHECK_STS(sts);
                 m_bCmCopy = true;
             }
         }else if(m_bCmCopy){
-            if(m_pCmCopy.get())
-                m_pCmCopy.get()->ReleaseCmSurfaces();
+            if(m_pCmCopy)
+                m_pCmCopy->ReleaseCmSurfaces();
             else
                 m_bCmCopy = false;
         }
@@ -575,6 +636,31 @@ VAAPIVideoCORE::AllocFrames(
 } // mfxStatus VAAPIVideoCORE::AllocFrames(...)
 
 
+mfxStatus VAAPIVideoCORE::ReallocFrame(mfxFrameSurface1 *surf)
+{
+    MFX_CHECK_NULL_PTR1(surf);
+
+    mfxMemId memid = surf->Data.MemId;
+
+    if (!(surf->Data.MemType & MFX_MEMTYPE_INTERNAL_FRAME &&
+        ((surf->Data.MemType & MFX_MEMTYPE_DXVA2_DECODER_TARGET)||
+         (surf->Data.MemType & MFX_MEMTYPE_DXVA2_PROCESSOR_TARGET))))
+        return MFX_ERR_MEMORY_ALLOC;
+
+    mfxFrameAllocator *pFrameAlloc = GetAllocatorAndMid(memid);
+   if (!pFrameAlloc)
+       return MFX_ERR_MEMORY_ALLOC;
+
+   mfxHDL srcHandle;
+   if (MFX_ERR_NONE == GetFrameHDL(surf->Data.MemId, &srcHandle))
+   {
+       VASurfaceID *va_surf = (VASurfaceID*)srcHandle;
+       return mfxDefaultAllocatorVAAPI::ReallocFrameHW(pFrameAlloc->pthis, surf, va_surf);
+   }
+
+    return MFX_ERR_MEMORY_ALLOC;
+}
+
 mfxStatus
 VAAPIVideoCORE::DefaultAllocFrames(
     mfxFrameAllocRequest* request,
@@ -631,68 +717,8 @@ VAAPIVideoCORE::CreateVA(
         !(request->Type & MFX_MEMTYPE_DXVA2_DECODER_TARGET))
         return MFX_ERR_NONE;
 
-    int profile = UMC::VA_VLD;
-
-    // video accelerator is needed for decoders only
-    switch (param->mfx.CodecId)
-    {
-    case MFX_CODEC_VC1:
-        profile |= VA_VC1;
-        break;
-    case MFX_CODEC_MPEG2:
-        profile |= VA_MPEG2;
-        break;
-    case MFX_CODEC_AVC:
-        profile |= VA_H264;
-        break;
-    case MFX_CODEC_HEVC:
-        profile |= VA_H265;
-        if (MFX_PROFILE_HEVC_REXT == param->mfx.CodecProfile)
-        {
-            profile |= VA_PROFILE_REXT;
-        }
-        if (param->mfx.FrameInfo.FourCC == MFX_FOURCC_P010
-#if (MFX_VERSION >= 1027)
-            || param->mfx.FrameInfo.FourCC == MFX_FOURCC_Y210
-            || param->mfx.FrameInfo.FourCC == MFX_FOURCC_Y410
-#endif
-        )
-        {
-            profile |= VA_PROFILE_10;
-        }
-
-        if (MFX_CHROMAFORMAT_YUV422 == param->mfx.FrameInfo.ChromaFormat)
-            profile |= (VA_PROFILE_422 | VA_PROFILE_REXT);
-        else if (MFX_CHROMAFORMAT_YUV444 == param->mfx.FrameInfo.ChromaFormat)
-            profile |= (VA_PROFILE_444 |VA_PROFILE_REXT);
-
-        break;
-    case MFX_CODEC_VP8:
-        profile |= VA_VP8;
-        break;
-    case MFX_CODEC_VP9:
-        profile |= VA_VP9;
-        switch (param->mfx.FrameInfo.FourCC)
-        {
-        case MFX_FOURCC_P010:
-            profile |= VA_PROFILE_10;
-            break;
-        case MFX_FOURCC_AYUV:
-            profile |= VA_PROFILE_444;
-            break;
-#if (MFX_VERSION >= 1027)
-        case MFX_FOURCC_Y410:
-            profile |= VA_PROFILE_10 | VA_PROFILE_444;
-            break;
-#endif
-        }
-        break;
-    case MFX_CODEC_JPEG:
-        profile |= VA_JPEG;
-        break;
-    default:
-        return MFX_ERR_UNSUPPORTED;
-    }
+    auto const profile = ChooseProfile(param, GetHWType());
+    MFX_CHECK(profile != UMC::UNKNOWN, MFX_ERR_UNSUPPORTED);
 
     bool init_render_targets =
 #if defined(ANDROID)
@@ -763,7 +789,7 @@ VAAPIVideoCORE::ProcessRenderTargets(
 #endif
 
     RegisterMids(response, request->Type, !m_bUseExtAllocForHWFrames, pAlloc);
-    m_pcHWAlloc.pop();
+    m_pcHWAlloc.release();
 
     return MFX_ERR_NONE;
 
@@ -787,19 +813,18 @@ VAAPIVideoCORE::GetVAService(
 
 } // mfxStatus VAAPIVideoCORE::GetVAService(...)
 
-mfxStatus
-VAAPIVideoCORE::SetCmCopyStatus(bool enable)
+void
+VAAPIVideoCORE::SetCmCopy(bool enable)
 {
     m_bCmCopyAllowed = enable;
     if (!enable)
     {
-        if (m_pCmCopy.get())
+        if (m_pCmCopy)
         {
-            m_pCmCopy.get()->Release();
+            m_pCmCopy->Release();
         }
         m_bCmCopy = false;
     }
-    return MFX_ERR_NONE;
 } // mfxStatus VAAPIVideoCORE::SetCmCopyStatus(...)
 
 mfxStatus
@@ -857,11 +882,11 @@ VAAPIVideoCORE::CreateVideoAccelerator(
     }
 
     m_pVA.reset((params.m_CreateFlags & VA_DECODE_STREAM_OUT_ENABLE) ? new FEIVideoAccelerator() : new LinuxVideoAccelerator());
-    m_pVA.get()->m_Platform = UMC::VA_LINUX;
-    m_pVA.get()->m_Profile = (VideoAccelerationProfile)profile;
-    m_pVA.get()->m_HWPlatform = m_HWType;
+    m_pVA->m_Platform = UMC::VA_LINUX;
+    m_pVA->m_Profile = (VideoAccelerationProfile)profile;
+    m_pVA->m_HWPlatform = m_HWType;
 
-    st = m_pVA.get()->Init(&params);
+    st = m_pVA->Init(&params);
 
     if(UMC_OK != st)
     {
@@ -1082,15 +1107,13 @@ VAAPIVideoCORE::DoFastCopyExtended(
         return MFX_ERR_UNDEFINED_BEHAVIOR;
     }
 
-    CmCopyWrapper *pCmCopy = m_pCmCopy.get();
-
     bool canUseCMCopy = m_bCmCopy ? CmCopyWrapper::CanUseCmCopy(pDst, pSrc) : false;
 
     if (NULL != pSrc->Data.MemId && NULL != pDst->Data.MemId)
     {
         if (canUseCMCopy)
         {
-            sts = pCmCopy->CopyVideoToVideo(pDst, pSrc);
+            sts = m_pCmCopy->CopyVideoToVideo(pDst, pSrc);
             MFX_CHECK_STS(sts);
         }
         else
@@ -1127,7 +1150,7 @@ VAAPIVideoCORE::DoFastCopyExtended(
         {
             if (canUseCMCopy)
             {
-                sts = pCmCopy->CopyVideoToSys(pDst, pSrc);
+                sts = m_pCmCopy->CopyVideoToSys(pDst, pSrc);
                 MFX_CHECK_STS(sts);
             }
             else
@@ -1187,7 +1210,7 @@ VAAPIVideoCORE::DoFastCopyExtended(
     {
         if (canUseCMCopy)
         {
-            sts = pCmCopy->CopySysToVideo(pDst, pSrc);
+            sts = m_pCmCopy->CopySysToVideo(pDst, pSrc);
             MFX_CHECK_STS(sts);
         }
         else
@@ -1261,6 +1284,7 @@ mfxStatus VAAPIVideoCORE::IsGuidSupported(const GUID guid,
 
     MFX_CHECK(m_Display, MFX_ERR_DEVICE_FAILED);
 
+#if VA_CHECK_VERSION(1, 2, 0)
     VaGuidMapper mapper(guid);
     VAProfile req_profile = mapper.profile;
     VAEntrypoint req_entrypoint = mapper.entrypoint;
@@ -1312,6 +1336,39 @@ mfxStatus VAAPIVideoCORE::IsGuidSupported(const GUID guid,
     MFX_CHECK(attr[1].value >= par->mfx.FrameInfo.Height, MFX_ERR_UNSUPPORTED);
 
     return MFX_ERR_NONE;
+#else
+    (void)guid;
+
+    switch (par->mfx.CodecId)
+    {
+    case MFX_CODEC_VC1:
+    case MFX_CODEC_AVC:
+    case MFX_CODEC_VP9:
+        break;
+    case MFX_CODEC_HEVC:
+        MFX_CHECK(m_HWType >= MFX_HW_HSW, MFX_WRN_PARTIAL_ACCELERATION);
+        MFX_CHECK(par->mfx.FrameInfo.Width <= 8192 && par->mfx.FrameInfo.Height <= 8192, MFX_WRN_PARTIAL_ACCELERATION);
+        break;
+    case MFX_CODEC_MPEG2: //MPEG2 decoder doesn't support resolution bigger than 2K
+        MFX_CHECK(par->mfx.FrameInfo.Width <= 2048 && par->mfx.FrameInfo.Height <= 2048, MFX_WRN_PARTIAL_ACCELERATION);
+        break;
+    case MFX_CODEC_JPEG:
+        MFX_CHECK(par->mfx.FrameInfo.Width <= 8192 && par->mfx.FrameInfo.Height <= 8192, MFX_WRN_PARTIAL_ACCELERATION);
+        break;
+    case MFX_CODEC_VP8:
+        MFX_CHECK(m_HWType >= MFX_HW_BDW, MFX_ERR_UNSUPPORTED);
+        break;
+    default:
+        return MFX_ERR_UNSUPPORTED;
+    }
+
+    MFX_CHECK(MFX_CODEC_JPEG == par->mfx.CodecId || MFX_CODEC_HEVC == par->mfx.CodecId ||
+        (par->mfx.FrameInfo.Width <= 4096 && par->mfx.FrameInfo.Height <= 4096),
+        MFX_WRN_PARTIAL_ACCELERATION
+    );
+
+    return MFX_ERR_NONE;
+#endif
 }
 
 void* VAAPIVideoCORE::QueryCoreInterface(const MFX_GUID &guid)
@@ -1354,35 +1411,35 @@ void* VAAPIVideoCORE::QueryCoreInterface(const MFX_GUID &guid)
         if (!m_bCmCopy)
         {
             m_pCmCopy.reset(new CmCopyWrapper);
-            pCmDevice = m_pCmCopy.get()->GetCmDevice(m_Display);
+            pCmDevice = m_pCmCopy->GetCmDevice(m_Display);
             if (!pCmDevice)
                 return NULL;
-            if (MFX_ERR_NONE != m_pCmCopy.get()->Initialize(GetHWType()))
+            if (MFX_ERR_NONE != m_pCmCopy->Initialize(GetHWType()))
                 return NULL;
             m_bCmCopy = true;
         }
         else
         {
-            pCmDevice =  m_pCmCopy.get()->GetCmDevice(m_Display);
+            pCmDevice =  m_pCmCopy->GetCmDevice(m_Display);
         }
         return (void*)pCmDevice;
     }
     else if (MFXICORECMCOPYWRAPPER_GUID == guid)
     {
-        if (!m_pCmCopy.get())
+        if (!m_pCmCopy)
         {
             m_pCmCopy.reset(new CmCopyWrapper);
-            if (!m_pCmCopy.get()->GetCmDevice(m_Display))
+            if (!m_pCmCopy->GetCmDevice(m_Display))
             {
                 m_bCmCopy = false;
                 m_bCmCopyAllowed = false;
-                m_pCmCopy.get()->Release();
+                m_pCmCopy->Release();
                 m_pCmCopy.reset();
                 return NULL;
             }
             else
             {
-                if (MFX_ERR_NONE != m_pCmCopy.get()->Initialize(GetHWType()))
+                if (MFX_ERR_NONE != m_pCmCopy->Initialize(GetHWType()))
                     return NULL;
                 else
                     m_bCmCopy = true;
@@ -1392,7 +1449,7 @@ void* VAAPIVideoCORE::QueryCoreInterface(const MFX_GUID &guid)
     }
     else if (MFXICMEnabledCore_GUID == guid)
     {
-        if (!m_pCmAdapter.get())
+        if (!m_pCmAdapter)
         {
             m_pCmAdapter.reset(new CMEnabledCoreAdapter(this));
         }
