@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2018 Intel Corporation
+// Copyright (c) 2017-2019 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -29,6 +29,7 @@
 #endif
 
 #include "umc_h265_tables.h"
+#include "umc_va_linux_protected.h"
 #include "mfx_ext_buffers.h"
 
 using namespace UMC;
@@ -76,13 +77,25 @@ namespace UMC_HEVC_DECODER
         g_sigLastScanCG32x32
     };
 
+#if defined(MFX_ENABLE_CPLIB)
+    extern Packer * CreatePackerCENC(VideoAccelerator*);
+#endif
 
-Packer * Packer::CreatePacker(UMC::VideoAccelerator * va)
+Packer * Packer::CreatePacker(VideoAccelerator * va)
 {
-    return new PackerVA{va};
+    Packer * packer = 0;
+
+#ifdef MFX_ENABLE_CPLIB
+    if (va->GetProtectedVA() && IS_PROTECTION_CENC(va->GetProtectedVA()->GetProtected()))
+        packer = CreatePackerCENC(va);
+    else
+#endif
+    packer = new PackerVA(va);
+
+    return packer;
 }
 
-Packer::Packer(UMC::VideoAccelerator * va)
+Packer::Packer(VideoAccelerator * va)
     : m_va(va)
 {
 }
@@ -99,12 +112,15 @@ void PackerVA::PackPicParams(const H265DecoderFrame *pCurrentFrame, TaskSupplier
 {
     H265DecoderFrameInfo const* pSliceInfo = pCurrentFrame->GetAU();
     if (!pSliceInfo)
-        throw h265_exception(UMC::UMC_ERR_FAILED);
+        throw h265_exception(UMC_ERR_FAILED);
 
     auto pSlice = pSliceInfo->GetSlice(0);
-    const H265SliceHeader* sliceHeader = pSlice->GetSliceHeader();
-    const H265SeqParamSet* pSeqParamSet = pSlice->GetSeqParam();
-    const H265PicParamSet* pPicParamSet = pSlice->GetPicParam();
+    if (!pSlice)
+        throw h265_exception(UMC_ERR_FAILED);
+
+    H265SliceHeader const* sliceHeader =  pSlice->GetSliceHeader();
+    H265SeqParamSet const* pSeqParamSet = pSlice->GetSeqParam();
+    H265PicParamSet const* pPicParamSet = pSlice->GetPicParam();
     VAPictureParameterBufferHEVC *picParam = nullptr;
 #if (MFX_VERSION >= 1027)
     VAPictureParameterBufferHEVCExtension* picParamExt = nullptr;
@@ -375,7 +391,7 @@ void PackerVA::CreateSliceDataBuffer(H265DecoderFrameInfo const* sliceInfo)
         size += NalUnitSize + 3;
     }
 
-    AlignedNalUnitSize = align_value<int32_t>(size, 128);
+    AlignedNalUnitSize = mfx::align2_value(size, 128);
 
     UMCVACompBuffer* compBuf;
     m_va->GetCompBuffer(VASliceDataBufferType, &compBuf, AlignedNalUnitSize);
@@ -412,20 +428,11 @@ void PackerVA::PackSliceParams(H265Slice const* pSlice, size_t sliceNum, bool is
         picParamsExt = (VAPictureParameterBufferHEVCExtension*)m_va->GetCompBuffer(VAPictureParameterBufferType);
         if (!picParamsExt)
             throw h265_exception(UMC_ERR_FAILED);
-        picParams = &picParamsExt->base;
-        if (!picParams)
-            throw h265_exception(UMC_ERR_FAILED);
 
         sliceParamsExt = (VASliceParameterBufferHEVCExtension*)m_va->GetCompBuffer(VASliceParameterBufferType, &compBuf);
 
         if (!sliceParamsExt)
             throw h265_exception(UMC_ERR_FAILED);
-        sliceParams = &sliceParamsExt->base;
-        if (!sliceParams)
-            throw h265_exception(UMC_ERR_FAILED);
-        sliceParamsRExt = &sliceParamsExt->rext;
-        if (!sliceParamsRExt)
-            throw h265_exception(UMC_ERR_FAILED);
 
         if (m_va->IsLongSliceControl())
         {
@@ -437,6 +444,18 @@ void PackerVA::PackSliceParams(H265Slice const* pSlice, size_t sliceNum, bool is
             sliceParamsExt = (VASliceParameterBufferHEVCExtension*)((VASliceParameterBufferBase*)sliceParamsExt + sliceNum);
             memset(sliceParamsExt, 0, sizeof(VASliceParameterBufferHEVCExtension));
         }
+
+        picParams = &picParamsExt->base;
+        if (!picParams)
+            throw h265_exception(UMC_ERR_FAILED);
+
+        sliceParams = &sliceParamsExt->base;
+        if (!sliceParams)
+            throw h265_exception(UMC_ERR_FAILED);
+
+        sliceParamsRExt = &sliceParamsExt->rext;
+        if (!sliceParamsRExt)
+            throw h265_exception(UMC_ERR_FAILED);
     }
     else
 #endif
@@ -701,19 +720,19 @@ void PackerVA::PackQmatrix(const H265Slice *pSlice)
 void PackerVA::PackAU(const H265DecoderFrame *frame, TaskSupplier_H265 * supplier)
 {
     if (!frame || !supplier)
-            throw h265_exception(UMC_ERR_NULL_PTR);
+        throw h265_exception(UMC_ERR_NULL_PTR);
 
-    H265DecoderFrameInfo const* sliceInfo = frame->m_pSlicesInfo;
-    size_t const sliceCount = sliceInfo->GetSliceCount();
-    if (!sliceCount)
-        return;
+    PackPicParams(frame, supplier);
 
-    H265Slice *pSlice = sliceInfo->GetSlice(0);
-    const H265SeqParamSet *pSeqParamSet = pSlice->GetSeqParam();
-    H265DecoderFrame *pCurrentFrame = pSlice->GetCurrentFrame();
+    H265DecoderFrameInfo const* sliceInfo = frame->GetAU();
+    if (!sliceInfo)
+        throw h265_exception(UMC_ERR_FAILED);
 
-    PackPicParams(pCurrentFrame, supplier);
+    auto pSlice = sliceInfo->GetSlice(0);
+    if (!pSlice)
+        throw h265_exception(UMC_ERR_FAILED);
 
+    H265SeqParamSet const* pSeqParamSet = pSlice->GetSeqParam();
     if (pSeqParamSet->scaling_list_enabled_flag)
     {
         PackQmatrix(pSlice);
@@ -722,11 +741,10 @@ void PackerVA::PackAU(const H265DecoderFrame *frame, TaskSupplier_H265 * supplie
     CreateSliceParamBuffer(sliceInfo);
     CreateSliceDataBuffer(sliceInfo);
 
-    size_t sliceNum = 0;
-    for (size_t n = 0; n < sliceCount; n++)
+    const size_t sliceCount = sliceInfo->GetSliceCount();
+    for (size_t sliceNum = 0; sliceNum < sliceCount; sliceNum++)
     {
-        PackSliceParams(sliceInfo->GetSlice(n), sliceNum, n == sliceCount - 1);
-        sliceNum++;
+        PackSliceParams(sliceInfo->GetSlice(sliceNum), sliceNum, sliceNum == sliceCount - 1);
     }
 
     Status s = m_va->Execute();
