@@ -1,4 +1,4 @@
-// Copyright (c) 2018 Intel Corporation
+// Copyright (c) 2018-2019 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -695,14 +695,6 @@ namespace UMC
                     ~(lAlignValue - 1));
     }
 
-    // template to align value
-    template<class T> inline
-    T align_value(size_t nValue, size_t lAlignValue = DEFAULT_ALIGN_VALUE)
-    {
-        return static_cast<T> ((nValue + (lAlignValue - 1)) &
-                               ~(lAlignValue - 1));
-    }
-
     // template to get minimum value
     template<class value_t, class min_t> inline
     value_t get_min(value_t value, min_t another)
