@@ -1,5 +1,5 @@
 /******************************************************************************\
-Copyright (c) 2005-2018, Intel Corporation
+Copyright (c) 2005-2019, Intel Corporation
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -21,12 +21,23 @@ or https://software.intel.com/en-us/media-client-solutions-support.
 #include "sample_defs.h"
 #include <algorithm>
 
+#if defined(_WIN32) || defined(_WIN64)
+#include <tchar.h>
+#include <windows.h>
+#endif
 
 #include <ctime>
 #include <algorithm>
+#include <thread>
 #include "pipeline_decode.h"
 #include "sysmem_allocator.h"
 
+#if defined(_WIN32) || defined(_WIN64)
+#include "d3d_allocator.h"
+#include "d3d11_allocator.h"
+#include "d3d_device.h"
+#include "d3d11_device.h"
+#endif
 
 #if defined LIBVA_SUPPORT
 #include "vaapi_allocator.h"
@@ -68,6 +79,8 @@ CDecodingPipeline::CDecodingPipeline()
     m_memType = SYSTEM_MEMORY;
     m_bExternalAlloc = false;
     m_bDecOutSysmem = false;
+    m_bSoftRobustFlag = false;
+
     MSDK_ZERO_MEMORY(m_mfxResponse);
     MSDK_ZERO_MEMORY(m_mfxVppResponse);
 
@@ -220,6 +233,7 @@ mfxStatus CDecodingPipeline::Init(sInputParams *pParams)
     m_bOutI420 = pParams->outI420;
 
     m_nTimeout = pParams->nTimeout;
+    m_bSoftRobustFlag = pParams->bSoftRobustFlag;
 
     // Initializing file reader
     totalBytesProcessed = 0;
@@ -382,7 +396,7 @@ mfxStatus CDecodingPipeline::Init(sInputParams *pParams)
             }
             if(sts==MFX_ERR_UNSUPPORTED)
             {
-                msdk_printf(isDefaultPlugin ?
+                msdk_printf(MSDK_STRING("%s"), isDefaultPlugin ?
                     MSDK_STRING("Default plugin cannot be loaded (possibly you have to define plugin explicitly)\n")
                     : MSDK_STRING("Explicitly specified plugin cannot be loaded.\n"));
             }
@@ -744,11 +758,19 @@ mfxStatus CDecodingPipeline::InitMfxParams(sInputParams *pParams)
         }
     }
 
-    // Only shifted P010 is supported now
-    if (m_mfxVideoParams.mfx.FrameInfo.FourCC == MFX_FOURCC_P010
+    // specify memory type
+    if (!m_bVppIsUsed)
+        m_mfxVideoParams.IOPattern = (mfxU16)(m_memType != SYSTEM_MEMORY ? MFX_IOPATTERN_OUT_VIDEO_MEMORY : MFX_IOPATTERN_OUT_SYSTEM_MEMORY);
+    else
+        m_mfxVideoParams.IOPattern = (mfxU16)(pParams->bUseHWLib ? MFX_IOPATTERN_OUT_VIDEO_MEMORY : MFX_IOPATTERN_OUT_SYSTEM_MEMORY);
+
+    // Only shifted P010 is supported now for video memory
+    if (m_mfxVideoParams.IOPattern != MFX_IOPATTERN_OUT_SYSTEM_MEMORY &&
+        (m_mfxVideoParams.mfx.FrameInfo.FourCC == MFX_FOURCC_P010
 #if (MFX_VERSION >= 1027)
         || m_mfxVideoParams.mfx.FrameInfo.FourCC == MFX_FOURCC_Y210
 #endif
+        )
     ) 
     {
         m_mfxVideoParams.mfx.FrameInfo.Shift = 1;
@@ -831,12 +853,6 @@ mfxStatus CDecodingPipeline::InitMfxParams(sInputParams *pParams)
         numViews = 1;
     }
 
-    // specify memory type
-    if (!m_bVppIsUsed)
-        m_mfxVideoParams.IOPattern = (mfxU16)(m_memType != SYSTEM_MEMORY ? MFX_IOPATTERN_OUT_VIDEO_MEMORY : MFX_IOPATTERN_OUT_SYSTEM_MEMORY);
-    else
-        m_mfxVideoParams.IOPattern = (mfxU16)(pParams->bUseHWLib ? MFX_IOPATTERN_OUT_VIDEO_MEMORY : MFX_IOPATTERN_OUT_SYSTEM_MEMORY);
-
     m_mfxVideoParams.AsyncDepth = pParams->nAsyncDepth;
 
     return MFX_ERR_NONE;
@@ -919,11 +935,13 @@ mfxStatus CDecodingPipeline::InitVppParams()
     }
 
     // P010 video surfaces should be shifted
-    if ((m_mfxVppVideoParams.vpp.Out.FourCC == MFX_FOURCC_P010 
+    if (m_memType != SYSTEM_MEMORY &&
+        (  m_mfxVppVideoParams.vpp.Out.FourCC == MFX_FOURCC_P010
 #if (MFX_VERSION >= 1027)
         || m_mfxVppVideoParams.vpp.Out.FourCC == MFX_FOURCC_Y210
 #endif
-        )&& m_memType != SYSTEM_MEMORY)
+        )
+    )
     {
         m_mfxVppVideoParams.vpp.Out.Shift = 1;
     }
@@ -1016,7 +1034,7 @@ mfxStatus CDecodingPipeline::AllocFrames()
     MSDK_CHECK_STATUS(sts, "m_pmfxDEC->Query failed");
 
     // Workaround for VP9 codec
-    if ((m_mfxVideoParams.mfx.FrameInfo.FourCC == MFX_FOURCC_P010 
+    if ((m_mfxVideoParams.mfx.FrameInfo.FourCC == MFX_FOURCC_P010
 #if (MFX_VERSION >= 1027)
         || m_mfxVideoParams.mfx.FrameInfo.FourCC == MFX_FOURCC_Y210
 #endif
@@ -1516,10 +1534,8 @@ mfxStatus CDecodingPipeline::DeliverOutput(mfxFrameSurface1* frame)
     return res;
 }
 
-mfxStatus CDecodingPipeline::DeliverLoop(void)
+void CDecodingPipeline::DeliverLoop(void)
 {
-    mfxStatus res = MFX_ERR_NONE;
-
     while (!m_bStopDeliverLoop) {
         m_pDeliverOutputSemaphore->Wait();
         if (m_bStopDeliverLoop) {
@@ -1542,17 +1558,7 @@ mfxStatus CDecodingPipeline::DeliverLoop(void)
         msdk_atomic_inc32(&m_output_count);
         m_pDeliveredEvent->Signal();
     }
-    return res;
-}
-
-unsigned int MFX_STDCALL CDecodingPipeline::DeliverThreadFunc(void* ctx)
-{
-    CDecodingPipeline* pipeline = (CDecodingPipeline*)ctx;
-
-    mfxStatus sts;
-    sts = pipeline->DeliverLoop();
-
-    return 0;
+    return;
 }
 
 void CDecodingPipeline::PrintPerFrameStat(bool force)
@@ -1593,7 +1599,7 @@ mfxStatus CDecodingPipeline::SyncOutputSurface(mfxU32 wait)
 
     mfxStatus sts = m_mfxSession.SyncOperation(m_pCurrentOutputSurface->syncp, wait);
 
-    if (MFX_ERR_GPU_HANG == sts) {
+    if (MFX_ERR_GPU_HANG == sts && m_bSoftRobustFlag) {
         msdk_printf(MSDK_STRING("GPU hang happened\n"));
         // Output surface can be corrupted
         // But should be delivered to output anyway
@@ -1642,8 +1648,8 @@ mfxStatus CDecodingPipeline::RunDecoding()
     mfxStatus           sts = MFX_ERR_NONE;
     bool                bErrIncompatibleVideoParams = false;
     CTimeInterval<>     decodeTimer(m_bIsCompleteFrame);
-    time_t start_time = time(0);
-    MSDKThread * pDeliverThread = NULL;
+    time_t              start_time = time(0);
+    std::thread         deliverThread;
 #if (MFX_VERSION >= 1025)
     mfxExtDecodeErrorReport *pDecodeErrorReport = NULL;
 #endif
@@ -1651,13 +1657,8 @@ mfxStatus CDecodingPipeline::RunDecoding()
     if (m_eWorkMode == MODE_RENDERING) {
         m_pDeliverOutputSemaphore = new MSDKSemaphore(sts);
         m_pDeliveredEvent = new MSDKEvent(sts, false, false);
-        pDeliverThread = new MSDKThread(sts, DeliverThreadFunc, this);
-        if (!pDeliverThread || !m_pDeliverOutputSemaphore || !m_pDeliveredEvent) {
-            MSDK_SAFE_DELETE(pDeliverThread);
-            MSDK_SAFE_DELETE(m_pDeliverOutputSemaphore);
-            MSDK_SAFE_DELETE(m_pDeliveredEvent);
-            return MFX_ERR_MEMORY_ALLOC;
-        }
+
+        deliverThread = std::thread (&CDecodingPipeline::DeliverLoop, this);
     }
 
     while (((sts == MFX_ERR_NONE) || (MFX_ERR_MORE_DATA == sts) || (MFX_ERR_MORE_SURFACE == sts)) && (m_nFrames > m_output_count))
@@ -1777,7 +1778,7 @@ mfxStatus CDecodingPipeline::RunDecoding()
                 if (pBitstream && MFX_ERR_MORE_DATA == sts && pBitstream->MaxLength == pBitstream->DataLength)
                 {
                     mfxStatus stsExt = ExtendMfxBitstream(pBitstream, pBitstream->MaxLength * 2);
-                    MSDK_CHECK_STATUS_SAFE(stsExt, "ExtendMfxBitstream failed", MSDK_SAFE_DELETE(pDeliverThread));
+                    MSDK_CHECK_STATUS(stsExt, "ExtendMfxBitstream failed");
                 }
 
                 if (MFX_WRN_DEVICE_BUSY == sts) {
@@ -1922,7 +1923,7 @@ mfxStatus CDecodingPipeline::RunDecoding()
 
     if (m_nFrames == m_output_count)
     {
-        if (!sts)
+        if (sts != MFX_ERR_NONE)
             msdk_printf(MSDK_STRING("[WARNING] Decoder returned error %s that could be compensated during next iterations of decoding process.\
                                     But requested amount of frames is already successfully decoded, so whole process is finished successfully."),
                         StatusToString(sts).c_str());
@@ -1949,13 +1950,13 @@ mfxStatus CDecodingPipeline::RunDecoding()
     if (m_eWorkMode == MODE_RENDERING) {
         m_bStopDeliverLoop = true;
         m_pDeliverOutputSemaphore->Post();
-        if (pDeliverThread)
-            pDeliverThread->Wait();
+
+        if (deliverThread.joinable())
+            deliverThread.join();
     }
 
     MSDK_SAFE_DELETE(m_pDeliverOutputSemaphore);
     MSDK_SAFE_DELETE(m_pDeliveredEvent);
-    MSDK_SAFE_DELETE(pDeliverThread);
 
     // exit in case of other errors
     MSDK_CHECK_STATUS(sts, "Unexpected error!!");
@@ -1974,11 +1975,11 @@ void CDecodingPipeline::PrintInfo()
     msdk_printf(MSDK_STRING("\nInput video\t%s\n"), CodecIdToStr(m_mfxVideoParams.mfx.CodecId).c_str());
     if (m_bVppIsUsed)
     {
-        msdk_printf(MSDK_STRING("Output format\t%s (using vpp)\n"), m_bOutI420 ? "I420(YUV)" : CodecIdToStr(m_mfxVppVideoParams.vpp.Out.FourCC).c_str());
+        msdk_printf(MSDK_STRING("Output format\t%s (using vpp)\n"), m_bOutI420 ? MSDK_STRING("I420(YUV)") : CodecIdToStr(m_mfxVppVideoParams.vpp.Out.FourCC).c_str());
     }
     else
     {
-        msdk_printf(MSDK_STRING("Output format\t%s\n"), m_bOutI420 ? "I420(YUV)" : CodecIdToStr(m_mfxVideoParams.mfx.FrameInfo.FourCC).c_str());
+        msdk_printf(MSDK_STRING("Output format\t%s\n"), m_bOutI420 ? MSDK_STRING("I420(YUV)") : CodecIdToStr(m_mfxVideoParams.mfx.FrameInfo.FourCC).c_str());
     }
 
     mfxFrameInfo Info = m_mfxVideoParams.mfx.FrameInfo;
@@ -2001,7 +2002,11 @@ void CDecodingPipeline::PrintInfo()
     msdk_printf(MSDK_STRING("Frame rate\t%.2f\n"), dFrameRate);
 
     const msdk_char* sMemType =
+#if defined(_WIN32) || defined(_WIN64)
+        m_memType == D3D9_MEMORY  ? MSDK_STRING("d3d")
+#else
         m_memType == D3D9_MEMORY  ? MSDK_STRING("vaapi")
+#endif
         : (m_memType == D3D11_MEMORY ? MSDK_STRING("d3d11")
         : MSDK_STRING("system"));
     msdk_printf(MSDK_STRING("Memory type\t\t%s\n"), sMemType);
