@@ -1,5 +1,5 @@
 /******************************************************************************\
-Copyright (c) 2005-2018, Intel Corporation
+Copyright (c) 2005-2019, Intel Corporation
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -69,8 +69,11 @@ void PrintHelp(msdk_char *strAppName, const msdk_char *strErrorMessage)
     msdk_printf(MSDK_STRING("Output format parameters:\n"));
     msdk_printf(MSDK_STRING("   [-i420] - pipeline output format: NV12, output file format: I420\n"));
     msdk_printf(MSDK_STRING("   [-nv12] - pipeline output format: NV12, output file format: NV12\n"));
+    msdk_printf(MSDK_STRING("   [-yuy2] - pipeline output format: YUV2, output file format: YUV2\n"));
+    msdk_printf(MSDK_STRING("   [-uyvy] - pipeline output format: UYVY, output file format: UYVY\n"));
     msdk_printf(MSDK_STRING("   [-rgb4] - pipeline output format: RGB4, output file format: RGB4\n"));
     msdk_printf(MSDK_STRING("   [-rgb4_fcr] - pipeline output format: RGB4 in full color range, output file format: RGB4 in full color range\n"));
+    msdk_printf(MSDK_STRING("   [-ayuv] - pipeline output format: AYUV, output file format: AYUV\n"));
     msdk_printf(MSDK_STRING("   [-p010] - pipeline output format: P010, output file format: P010\n"));
     msdk_printf(MSDK_STRING("   [-a2rgb10] - pipeline output format: A2RGB10, output file format: A2RGB10\n"));
     msdk_printf(MSDK_STRING("\n"));
@@ -106,6 +109,7 @@ void PrintHelp(msdk_char *strAppName, const msdk_char *strErrorMessage)
     msdk_printf(MSDK_STRING("   [-calc_latency]           - calculates latency during decoding and prints log (supported only for H.264 and JPEG codec)\n"));
     msdk_printf(MSDK_STRING("   [-async]                  - depth of asynchronous pipeline. default value is 4. must be between 1 and 20\n"));
     msdk_printf(MSDK_STRING("   [-gpucopy::<on,off>] Enable or disable GPU copy mode\n"));
+    msdk_printf(MSDK_STRING("   [-robust:soft]            - GPU hang recovery by inserting an IDR frame\n"));
     msdk_printf(MSDK_STRING("   [-timeout]                - timeout in seconds\n"));
 #if MFX_VERSION >= 1022
     msdk_printf(MSDK_STRING("   [-dec_postproc force/auto] - resize after decoder using direct pipe\n"));
@@ -115,11 +119,20 @@ void PrintHelp(msdk_char *strAppName, const msdk_char *strErrorMessage)
     msdk_printf(MSDK_STRING("                        or perform VPP operation through separate pipeline component for unsupported streams\n"));
 
 #endif //MFX_VERSION >= 1022
+#if !defined(_WIN32) && !defined(_WIN64)
     msdk_printf(MSDK_STRING("   [-threads_num]            - number of mediasdk task threads\n"));
     msdk_printf(MSDK_STRING("   [-threads_schedtype]      - scheduling type of mediasdk task threads\n"));
     msdk_printf(MSDK_STRING("   [-threads_priority]       - priority of mediasdk task threads\n"));
     msdk_printf(MSDK_STRING("\n"));
     msdk_thread_printf_scheduling_help();
+#endif
+#if defined(_WIN32) || defined(_WIN64)
+    msdk_printf(MSDK_STRING("   [-jpeg_rotate n]          - rotate jpeg frame n degrees \n"));
+    msdk_printf(MSDK_STRING("       n(90,180,270)         - number of degrees \n"));
+
+    msdk_printf(MSDK_STRING("\nFeatures: \n"));
+    msdk_printf(MSDK_STRING("   Press 1 to toggle fullscreen rendering on/off\n"));
+#endif
     msdk_printf(MSDK_STRING("\n"));
     msdk_printf(MSDK_STRING("Example:\n"));
     msdk_printf(MSDK_STRING("  %s h265 -i in.bit -o out.yuv -p 15dd936825ad475ea34e35f3f54217a6\n"), strAppName);
@@ -399,6 +412,11 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
         {
             pParams->gpuCopy = MFX_GPUCOPY_OFF;
         }
+        else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-robust:soft")))
+        {
+            pParams->bSoftRobustFlag = true;
+        }
+#if !defined(_WIN32) && !defined(_WIN64)
 #if (MFX_VERSION >= 1025)
         else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-d")))
         {
@@ -444,6 +462,7 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
                 return MFX_ERR_UNSUPPORTED;
             }
         }
+#endif // #if !defined(_WIN32) && !defined(_WIN64)
 #if MFX_VERSION >= 1022
         else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-dec_postproc")))
         {
@@ -545,6 +564,18 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
         {
             pParams->fourcc = MFX_FOURCC_RGB4;
         }
+        else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-ayuv")))
+        {
+            pParams->fourcc = MFX_FOURCC_AYUV;
+        }
+        else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-yuy2")))
+        {
+            pParams->fourcc = MFX_FOURCC_YUY2;
+        }
+        else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-uyvy")))
+        {
+            pParams->fourcc = MFX_FOURCC_UYVY;
+        }
         else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-rgb4_fcr")))
         {
             pParams->fourcc = MFX_FOURCC_RGB4;
@@ -644,7 +675,11 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
     return MFX_ERR_NONE;
 }
 
+#if defined(_WIN32) || defined(_WIN64)
+int _tmain(int argc, TCHAR *argv[])
+#else
 int main(int argc, char *argv[])
+#endif
 {
     sInputParams        Params;   // input parameters from command line
     CDecodingPipeline   Pipeline; // pipeline for decoding, includes input file reader, decoder and output file writer
