@@ -1,5 +1,5 @@
 /******************************************************************************\
-Copyright (c) 2005-2018, Intel Corporation
+Copyright (c) 2005-2019, Intel Corporation
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -79,6 +79,7 @@ CEncTaskPool::CEncTaskPool()
     m_pmfxSession       = NULL;
     m_nTaskBufferStart  = 0;
     m_nPoolSize         = 0;
+    m_bGpuHangRecovery = false;
 }
 
 CEncTaskPool::~CEncTaskPool()
@@ -139,9 +140,17 @@ mfxStatus CEncTaskPool::SynchronizeFirstTask()
     if (NULL != m_pTasks[m_nTaskBufferStart].EncSyncP)
     {
         sts = m_pmfxSession->SyncOperation(m_pTasks[m_nTaskBufferStart].EncSyncP, MSDK_WAIT_INTERVAL);
-        if (sts == MFX_ERR_GPU_HANG)
+        if (sts == MFX_ERR_GPU_HANG && m_bGpuHangRecovery)
         {
             bGpuHang = true;
+            {
+            for (mfxU32 i = 0; i < m_nPoolSize; i++)
+                if (m_pTasks[i].EncSyncP != NULL)
+                {
+                    sts = m_pmfxSession->SyncOperation(m_pTasks[i].EncSyncP, 0);//MSDK_WAIT_INTERVAL
+                }
+            }
+            ClearTasks();
             sts = MFX_ERR_NONE;
             msdk_printf(MSDK_STRING("GPU hang happened\n"));
         }
@@ -174,9 +183,10 @@ mfxStatus CEncTaskPool::SynchronizeFirstTask()
             {
                 // find out if the error occurred in a VPP task to perform recovery procedure if applicable
                 sts = m_pmfxSession->SyncOperation(*m_pTasks[m_nTaskBufferStart].DependentVppTasks.begin(), 0);
-                if (sts == MFX_ERR_GPU_HANG)
+                if (sts == MFX_ERR_GPU_HANG && m_bGpuHangRecovery)
                 {
                     bGpuHang = true;
+                    ClearTasks();
                     sts = MFX_ERR_NONE;
                     msdk_printf(MSDK_STRING("GPU hang happened\n"));
                 }
@@ -259,6 +269,20 @@ void CEncTaskPool::Close()
     m_nPoolSize = 0;
 }
 
+void CEncTaskPool::SetGpuHangRecoveryFlag()
+{
+    m_bGpuHangRecovery = true;
+}
+
+void CEncTaskPool::ClearTasks()
+{
+    for (size_t i = 0; i < m_nPoolSize; i++)
+    {
+        m_pTasks[i].Reset();
+    }
+    m_nTaskBufferStart = 0;
+}
+
 sTask::sTask()
     : EncSyncP(0)
     , pWriter(NULL)
@@ -471,6 +495,14 @@ mfxStatus CEncodingPipeline::InitMfxEncParams(sInputParams *pInParams)
     m_mfxEncParams.mfx.FrameInfo.CropW = pInParams->nDstWidth;
     m_mfxEncParams.mfx.FrameInfo.CropH = pInParams->nDstHeight;
 
+    m_ExtHEVCTiles.NumTileRows = pInParams->nEncTileRows;
+    m_ExtHEVCTiles.NumTileColumns = pInParams->nEncTileCols;
+
+#if MFX_VERSION >= MFX_VERSION_NEXT
+    m_ExtVP9Param.NumTileRows    = pInParams->nEncTileRows;
+    m_ExtVP9Param.NumTileColumns = pInParams->nEncTileCols;
+#endif
+
     bool bCodingOption = false;
     if(*pInParams->uSEI && (pInParams->CodecId == MFX_CODEC_AVC ||
                 pInParams->CodecId == MFX_CODEC_HEVC))
@@ -553,15 +585,26 @@ mfxStatus CEncodingPipeline::InitMfxEncParams(sInputParams *pInParams)
         m_CodingOption.ViewOutput = MFX_CODINGOPTION_ON;
         bCodingOption = true;
     }
+
+    if (pInParams->nPicTimingSEI || pInParams->nNalHrdConformance || pInParams->nVuiNalHrdParameters)
+    {
+		m_CodingOption.PicTimingSEI = pInParams->nPicTimingSEI;
+		m_CodingOption.NalHrdConformance = pInParams->nNalHrdConformance;
+		m_CodingOption.VuiNalHrdParameters = pInParams->nVuiNalHrdParameters;
+                bCodingOption = true;
+    }
+
     if (bCodingOption)
     {
         m_EncExtParams.push_back((mfxExtBuffer *)&m_CodingOption);
     }
 
+
     // configure the depth of the look ahead BRC if specified in command line
     if (pInParams->nLADepth || pInParams->nMaxSliceSize || pInParams->nMaxFrameSize || pInParams->nBRefType ||
         (pInParams->nExtBRC && (pInParams->CodecId == MFX_CODEC_HEVC || pInParams->CodecId == MFX_CODEC_AVC)) ||
-        pInParams->IntRefType || pInParams->IntRefCycleSize || pInParams->IntRefQPDelta )
+        pInParams->IntRefType || pInParams->IntRefCycleSize || pInParams->IntRefQPDelta ||
+        pInParams->AdaptiveI || pInParams->AdaptiveB)
     {
         m_CodingOption2.LookAheadDepth = pInParams->nLADepth;
         m_CodingOption2.MaxSliceSize   = pInParams->nMaxSliceSize;
@@ -580,6 +623,8 @@ mfxStatus CEncodingPipeline::InitMfxEncParams(sInputParams *pInParams)
         m_CodingOption2.IntRefType = pInParams->IntRefType;
         m_CodingOption2.IntRefCycleSize = pInParams->IntRefCycleSize;
         m_CodingOption2.IntRefQPDelta = pInParams->IntRefQPDelta;
+        m_CodingOption2.AdaptiveI = pInParams->AdaptiveI;
+        m_CodingOption2.AdaptiveB = pInParams->AdaptiveB;
         m_EncExtParams.push_back((mfxExtBuffer *)&m_CodingOption2);
     }
 
@@ -597,14 +642,19 @@ mfxStatus CEncodingPipeline::InitMfxEncParams(sInputParams *pInParams)
         || pInParams->nPRefType || pInParams->IntRefCycleDist || pInParams->nAdaptiveMaxFrameSize
         || pInParams->nNumRefActiveP || pInParams->nNumRefActiveBL0 || pInParams->nNumRefActiveBL1
         || pInParams->ExtBrcAdaptiveLTR || pInParams->QVBRQuality || pInParams->WinBRCSize
-        || pInParams->WinBRCMaxAvgKbps)
-    {
+#if (MFX_VERSION >= MFX_VERSION_NEXT)
+        || pInParams->DeblockingAlphaTcOffset || pInParams->DeblockingBetaOffset
+#endif
+        || pInParams->WinBRCMaxAvgKbps || pInParams->nTransformSkip)    {
         if (pInParams->CodecId == MFX_CODEC_HEVC)
         {
             m_CodingOption3.GPB = pInParams->nGPB;
             std::fill(m_CodingOption3.NumRefActiveP,   m_CodingOption3.NumRefActiveP + 8,   pInParams->nNumRefActiveP);
             std::fill(m_CodingOption3.NumRefActiveBL0, m_CodingOption3.NumRefActiveBL0 + 8, pInParams->nNumRefActiveBL0);
             std::fill(m_CodingOption3.NumRefActiveBL1, m_CodingOption3.NumRefActiveBL1 + 8, pInParams->nNumRefActiveBL1);
+            #if (MFX_VERSION >= 1026)
+                m_CodingOption3.TransformSkip = pInParams->nTransformSkip;
+            #endif
         }
 
         m_CodingOption3.WeightedPred   = pInParams->WeightedPred;
@@ -622,9 +672,47 @@ mfxStatus CEncodingPipeline::InitMfxEncParams(sInputParams *pInParams)
         m_CodingOption3.WinBRCSize = pInParams->WinBRCSize;
         m_CodingOption3.WinBRCMaxAvgKbps = pInParams->WinBRCMaxAvgKbps;
 
+#if (MFX_VERSION >= MFX_VERSION_NEXT)
+        if (pInParams->DeblockingAlphaTcOffset || pInParams->DeblockingBetaOffset)
+        {
+            m_CodingOption3.DeblockingAlphaTcOffset = pInParams->DeblockingAlphaTcOffset;
+            m_CodingOption3.DeblockingBetaOffset = pInParams->DeblockingBetaOffset;
+        }
+#endif
         m_EncExtParams.push_back((mfxExtBuffer *)&m_CodingOption3);
     }
 
+        if (pInParams->nAvcTemp)
+	{
+		if (pInParams->CodecId == MFX_CODEC_HEVC)
+		{
+			m_AvcTemporalLayers.BaseLayerPID = pInParams->nBaseLayerPID;
+			for (int i = 0; i < 8; i++)
+			{
+				m_AvcTemporalLayers.Layer[i].Scale = pInParams->nAvcTemporalLayers[i];
+		   	}
+		        m_EncExtParams.push_back((mfxExtBuffer *)&m_AvcTemporalLayers);
+                }
+	}
+
+	if (pInParams->nSPSId || pInParams->nPPSId)
+	{
+		if (pInParams->CodecId == MFX_CODEC_HEVC)
+		{
+			m_CodingOptionSPSPPS.SPSId = pInParams->nSPSId;
+			m_CodingOptionSPSPPS.PPSId = pInParams->nPPSId;
+		}
+	        m_EncExtParams.push_back((mfxExtBuffer *)&m_CodingOptionSPSPPS);
+	}
+
+
+    if (m_ExtHEVCTiles.NumTileRows
+        && m_ExtHEVCTiles.NumTileColumns
+        && m_mfxEncParams.mfx.CodecId == MFX_CODEC_HEVC)
+    {
+        m_EncExtParams.push_back((mfxExtBuffer*)&m_ExtHEVCTiles);
+    }
+
     // In case of HEVC when height and/or width divided with 8 but not divided with 16
     // add extended parameter to increase performance
     if ( ( !((m_mfxEncParams.mfx.FrameInfo.CropW & 15 ) ^ 8 ) ||
@@ -636,6 +724,15 @@ mfxStatus CEncodingPipeline::InitMfxEncParams(sInputParams *pInParams)
         m_EncExtParams.push_back((mfxExtBuffer*)&m_ExtHEVCParam);
     }
 
+#if MFX_VERSION >= 1029
+    if (m_ExtVP9Param.NumTileRows
+        && m_ExtVP9Param.NumTileColumns
+        && m_mfxEncParams.mfx.CodecId == MFX_CODEC_VP9)
+    {
+        m_EncExtParams.push_back((mfxExtBuffer*)&m_ExtVP9Param);
+    }
+#endif
+
     if (pInParams->TransferMatrix)
     {
         m_VideoSignalInfo.ColourDescriptionPresent = 1;
@@ -955,7 +1052,7 @@ mfxStatus CEncodingPipeline::CreateAllocator()
             D3D11AllocatorParams *pd3dAllocParams = new D3D11AllocatorParams;
             MSDK_CHECK_POINTER(pd3dAllocParams, MFX_ERR_MEMORY_ALLOC);
             pd3dAllocParams->pDevice = reinterpret_cast<ID3D11Device *>(hdl);
-
+            pd3dAllocParams->bUseSingleTexture = m_bSingleTexture;
             m_pmfxAllocatorParams = pd3dAllocParams;
         }
         else
@@ -1095,6 +1192,9 @@ CEncodingPipeline::CEncodingPipeline()
     m_nFramesRead = 0;
     m_bFileWriterReset = false;
 
+    m_bSoftRobustFlag = false;
+    m_bSingleTexture = false;
+
     m_MVCflags = MVC_DISABLED;
     m_nNumView = 0;
 
@@ -1120,10 +1220,26 @@ CEncodingPipeline::CEncodingPipeline()
     m_CodingOption3.Header.BufferId = MFX_EXTBUFF_CODING_OPTION3;
     m_CodingOption3.Header.BufferSz = sizeof(m_CodingOption3);
 
+    MSDK_ZERO_MEMORY(m_AvcTemporalLayers);
+    m_AvcTemporalLayers.Header.BufferId = MFX_EXTBUFF_AVC_TEMPORAL_LAYERS;
+    m_AvcTemporalLayers.Header.BufferSz = sizeof(m_AvcTemporalLayers);
+
+    MSDK_ZERO_MEMORY(m_CodingOptionSPSPPS);
+    m_CodingOptionSPSPPS.Header.BufferId = MFX_EXTBUFF_CODING_OPTION_SPSPPS;
+    m_CodingOptionSPSPPS.Header.BufferSz = sizeof(m_CodingOptionSPSPPS);
+
     MSDK_ZERO_MEMORY(m_ExtHEVCParam);
     m_ExtHEVCParam.Header.BufferId = MFX_EXTBUFF_HEVC_PARAM;
     m_ExtHEVCParam.Header.BufferSz = sizeof(m_ExtHEVCParam);
 
+    MSDK_ZERO_MEMORY(m_ExtHEVCTiles);
+    m_ExtHEVCTiles.Header.BufferId = MFX_EXTBUFF_HEVC_TILES;
+    m_ExtHEVCTiles.Header.BufferSz = sizeof(m_ExtHEVCTiles);
+
+    MSDK_ZERO_MEMORY(m_ExtVP9Param);
+    m_ExtVP9Param.Header.BufferId = MFX_EXTBUFF_VP9_PARAM;
+    m_ExtVP9Param.Header.BufferSz = sizeof(m_ExtVP9Param);
+
     MSDK_ZERO_MEMORY(m_VideoSignalInfo);
     m_VideoSignalInfo.Header.BufferId = MFX_EXTBUFF_VIDEO_SIGNAL_INFO;
     m_VideoSignalInfo.Header.BufferSz = sizeof(m_VideoSignalInfo);
@@ -1264,6 +1380,7 @@ mfxStatus CEncodingPipeline::Init(sInputParams *pParams)
     initPar.Version.Minor = 0;
 
     initPar.GPUCopy = pParams->gpuCopy;
+    m_bSingleTexture = pParams->bSingleTexture;
 
     // Init session
     if (pParams->bUseHWLib) {
@@ -1341,7 +1458,7 @@ mfxStatus CEncodingPipeline::Init(sInputParams *pParams)
             }
             if (sts == MFX_ERR_UNSUPPORTED)
             {
-                msdk_printf(isDefaultPlugin ?
+                msdk_printf(MSDK_STRING("%s"), isDefaultPlugin ?
                     MSDK_STRING("Default plugin cannot be loaded (possibly you have to define plugin explicitly)\n")
                     : MSDK_STRING("Explicitly specified plugin cannot be loaded.\n"));
             }
@@ -1383,7 +1500,8 @@ mfxStatus CEncodingPipeline::Init(sInputParams *pParams)
     }
     // Determine if we should shift P010 surfaces
     bool readerShift = false;
-    if (pParams->FileInputFourCC == MFX_FOURCC_P010 || pParams->FileInputFourCC == MFX_FOURCC_P210 
+    if (pParams->FileInputFourCC == MFX_FOURCC_P010
+        || pParams->FileInputFourCC == MFX_FOURCC_P210
 #if (MFX_VERSION >= 1027)
         || pParams->FileInputFourCC == MFX_FOURCC_Y210
 #endif
@@ -1431,6 +1549,8 @@ mfxStatus CEncodingPipeline::Init(sInputParams *pParams)
     m_memType = pParams->memType;
     m_nMemBuffer = pParams->nMemBuf;
 
+    m_bSoftRobustFlag = pParams->bSoftRobustFlag;
+
     // create and init frame allocator
     sts = CreateAllocator();
     MSDK_CHECK_STATUS(sts, "CreateAllocator failed");
@@ -1752,6 +1872,9 @@ mfxStatus CEncodingPipeline::ResetMFXComponents(sInputParams* pParams)
     sts = m_TaskPool.Init(&m_mfxSession, m_FileWriters.first, m_mfxEncParams.AsyncDepth, nEncodedDataBufferSize, m_FileWriters.second);
     MSDK_CHECK_STATUS(sts, "m_TaskPool.Init failed");
 
+    if (m_bSoftRobustFlag)
+        m_TaskPool.SetGpuHangRecoveryFlag();
+
     sts = FillBuffers();
     MSDK_CHECK_STATUS(sts, "FillBuffers failed");
 
@@ -1860,8 +1983,10 @@ mfxStatus CEncodingPipeline::GetFreeTask(sTask **ppTask)
     if (MFX_ERR_NOT_FOUND == sts)
     {
         sts = m_TaskPool.SynchronizeFirstTask();
-        if (sts == MFX_ERR_GPU_HANG)
+        if (sts == MFX_ERR_GPU_HANG && m_bSoftRobustFlag)
         {
+            m_TaskPool.ClearTasks();
+            FreeSurfacePool(m_pEncSurfaces, m_EncResponse.NumFrameActual);
             m_bInsertIDR = true;
             sts = MFX_ERR_NONE;
         }
@@ -2218,9 +2343,11 @@ mfxStatus CEncodingPipeline::Run()
     while (MFX_ERR_NONE == sts)
     {
         sts = m_TaskPool.SynchronizeFirstTask();
-        if (sts == MFX_ERR_GPU_HANG)
+        if (sts == MFX_ERR_GPU_HANG && m_bSoftRobustFlag)
         {
             m_bInsertIDR = true;
+            m_TaskPool.ClearTasks();//may be not needed
+            FreeSurfacePool(m_pEncSurfaces, m_EncResponse.NumFrameActual);
             sts = MFX_ERR_NONE;
         }
     }
@@ -2326,7 +2453,11 @@ void CEncodingPipeline::PrintInfo()
     msdk_printf(MSDK_STRING("Target usage\t%s\n"), TargetUsageToStr(m_mfxEncParams.mfx.TargetUsage));
 
     const msdk_char* sMemType =
+#if defined(_WIN32) || defined(_WIN64)
+        m_memType == D3D9_MEMORY  ? MSDK_STRING("d3d")
+#else
         m_memType == D3D9_MEMORY  ? MSDK_STRING("vaapi")
+#endif
         : (m_memType == D3D11_MEMORY ? MSDK_STRING("d3d11")
         : MSDK_STRING("system"));
     msdk_printf(MSDK_STRING("Memory type\t%s\n"), sMemType);
